@@ -1,18 +1,18 @@
 //! Module implementing custom syntax for [`Engine`].
 
 use crate::ast::Expr;
-use crate::dynamic::Variant;
 use crate::engine::EvalContext;
-use crate::fn_native::SendSync;
+use crate::func::native::SendSync;
 use crate::r#unsafe::unsafe_try_cast;
-use crate::token::{is_valid_identifier, Token};
+use crate::tokenizer::{is_valid_identifier, Token};
+use crate::types::dynamic::Variant;
 use crate::{
     Engine, Identifier, ImmutableString, LexError, ParseError, Position, RhaiResult, Shared,
     StaticVec, INT,
 };
-use std::any::TypeId;
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
+use std::{any::TypeId, ops::Deref};
 
 /// Collection of special markers for custom syntax definition.
 pub mod markers {
@@ -33,6 +33,8 @@ pub mod markers {
     pub const CUSTOM_SYNTAX_MARKER_FLOAT: &str = "$float$";
     /// Special marker for matching a boolean value.
     pub const CUSTOM_SYNTAX_MARKER_BOOL: &str = "$bool$";
+    /// Special marker for identifying the custom syntax variant.
+    pub const CUSTOM_SYNTAX_MARKER_SYNTAX_VARIANT: &str = "$$";
 }
 
 /// A general expression evaluation trait object.
@@ -69,12 +71,6 @@ impl Expression<'_> {
     pub fn get_variable_name(&self) -> Option<&str> {
         self.0.get_variable_name(true)
     }
-    /// Get the expression.
-    #[inline(always)]
-    #[must_use]
-    pub(crate) const fn expr(&self) -> &Expr {
-        &self.0
-    }
     /// Get the position of this expression.
     #[inline(always)]
     #[must_use]
@@ -132,6 +128,22 @@ impl Expression<'_> {
     }
 }
 
+impl AsRef<Expr> for Expression<'_> {
+    #[inline(always)]
+    fn as_ref(&self) -> &Expr {
+        &self.0
+    }
+}
+
+impl Deref for Expression<'_> {
+    type Target = Expr;
+
+    #[inline(always)]
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
 impl EvalContext<'_, '_, '_, '_, '_, '_, '_, '_> {
     /// Evaluate an [expression tree][Expression].
     ///
@@ -146,7 +158,7 @@ impl EvalContext<'_, '_, '_, '_, '_, '_, '_, '_> {
             self.state,
             self.lib,
             self.this_ptr,
-            expr.expr(),
+            expr,
             self.level,
         )
     }
@@ -333,7 +345,6 @@ impl Engine {
     /// * `Ok(None)`: parsing complete and there are no more symbols to match.
     /// * `Ok(Some(symbol))`: the next symbol to match, which can also be `$expr$`, `$ident$` or `$block$`.
     /// * `Err(ParseError)`: error that is reflected back to the [`Engine`], normally `ParseError(ParseErrorType::BadInput(LexError::ImproperSymbol(message)), Position::NONE)` to indicate a syntax error, but it can be any [`ParseError`].
-    ///
     pub fn register_custom_syntax_raw(
         &mut self,
         key: impl Into<Identifier>,
