@@ -1,15 +1,16 @@
 //! Module defining external-loaded modules for Rhai.
 
 use crate::ast::{FnAccess, Ident};
-use crate::dynamic::Variant;
-use crate::fn_call::FnCallArgs;
-use crate::fn_native::{shared_take_or_clone, CallableFunction, IteratorFn, SendSync};
-use crate::fn_register::RegisterNativeFunction;
-use crate::parse::IdentifierBuilder;
-use crate::token::Token;
+use crate::func::{
+    shared_take_or_clone, CallableFunction, FnCallArgs, IteratorFn, RegisterNativeFunction,
+    SendSync,
+};
+use crate::parser::IdentifierBuilder;
+use crate::tokenizer::Token;
+use crate::types::dynamic::Variant;
 use crate::{
     calc_fn_params_hash, calc_qualified_fn_hash, combine_hashes, Dynamic, EvalAltResult,
-    Identifier, ImmutableString, NativeCallContext, Position, Shared, StaticVec,
+    Identifier, ImmutableString, NativeCallContext, Shared, StaticVec,
 };
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
@@ -22,12 +23,6 @@ use std::{
     ops::{Add, AddAssign, Deref, DerefMut},
 };
 
-#[cfg(not(feature = "no_index"))]
-use crate::Array;
-
-#[cfg(not(feature = "no_object"))]
-use crate::Map;
-
 /// A type representing the namespace of a function.
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
 pub enum FnNamespace {
@@ -37,13 +32,6 @@ pub enum FnNamespace {
     Internal,
 }
 
-impl Default for FnNamespace {
-    #[inline(always)]
-    fn default() -> Self {
-        Self::Internal
-    }
-}
-
 /// Data structure containing a single registered function.
 #[derive(Debug, Clone)]
 pub struct FuncInfo {
@@ -73,11 +61,11 @@ impl FuncInfo {
         let mut sig = format!("{}(", self.name);
 
         if !self.param_names.is_empty() {
-            let mut params: StaticVec<String> =
+            let mut params: StaticVec<Box<str>> =
                 self.param_names.iter().map(|s| s.as_str().into()).collect();
             let return_type = params.pop().unwrap_or_else(|| "()".into());
             sig.push_str(&params.join(", "));
-            if return_type != "()" {
+            if &*return_type != "()" {
                 sig.push_str(") -> ");
                 sig.push_str(&return_type);
             } else {
@@ -102,8 +90,7 @@ impl FuncInfo {
     }
 }
 
-/// _(internals)_ Calculate a [`u64`] hash key from a namespace-qualified function name and
-/// parameter types.
+/// _(internals)_ Calculate a non-zero [`u64`] hash key from a namespace-qualified function name and parameter types.
 /// Exported under the `internals` feature only.
 ///
 /// Module names are passed in via `&str` references from an iterator.
@@ -113,12 +100,12 @@ impl FuncInfo {
 ///
 /// The first module name is skipped.  Hashing starts from the _second_ module in the chain.
 #[inline]
-fn calc_native_fn_hash<'a>(
-    modules: impl Iterator<Item = &'a str>,
-    fn_name: &str,
+pub fn calc_native_fn_hash(
+    modules: impl Iterator<Item = impl AsRef<str>>,
+    fn_name: impl AsRef<str>,
     params: &[TypeId],
 ) -> u64 {
-    let hash_script = calc_qualified_fn_hash(modules, fn_name, params.len());
+    let hash_script = calc_qualified_fn_hash(modules, fn_name.as_ref(), params.len());
     let hash_params = calc_fn_params_hash(params.iter().cloned());
     combine_hashes(hash_script, hash_params)
 }
@@ -131,6 +118,8 @@ pub struct Module {
     id: Option<Identifier>,
     /// Is this module internal?
     pub(crate) internal: bool,
+    /// Is this module part of a standard library?
+    pub(crate) standard: bool,
     /// Sub-modules.
     modules: BTreeMap<Identifier, Shared<Module>>,
     /// [`Module`] variables.
@@ -184,8 +173,7 @@ impl fmt::Debug for Module {
             d.field(
                 "functions",
                 &self
-                    .functions
-                    .values()
+                    .iter_fn()
                     .map(|f| f.func.to_string())
                     .collect::<BTreeSet<_>>(),
             );
@@ -194,17 +182,10 @@ impl fmt::Debug for Module {
     }
 }
 
-impl AsRef<Module> for Module {
-    #[inline(always)]
-    fn as_ref(&self) -> &Module {
-        self
-    }
-}
-
 impl<M: AsRef<Module>> Add<M> for &Module {
     type Output = Module;
 
-    #[inline(always)]
+    #[inline]
     fn add(self, rhs: M) -> Self::Output {
         let mut module = self.clone();
         module.merge(rhs.as_ref());
@@ -240,22 +221,23 @@ impl Module {
     /// module.set_var("answer", 42_i64);
     /// assert_eq!(module.get_var_value::<i64>("answer").expect("answer should exist"), 42);
     /// ```
-    #[inline(always)]
+    #[inline]
     #[must_use]
     pub fn new() -> Self {
         Self {
             id: None,
             internal: false,
-            modules: Default::default(),
-            variables: Default::default(),
-            all_variables: Default::default(),
-            functions: Default::default(),
-            all_functions: Default::default(),
-            type_iterators: Default::default(),
-            all_type_iterators: Default::default(),
+            standard: false,
+            modules: BTreeMap::new(),
+            variables: BTreeMap::new(),
+            all_variables: BTreeMap::new(),
+            functions: BTreeMap::new(),
+            all_functions: BTreeMap::new(),
+            type_iterators: BTreeMap::new(),
+            all_type_iterators: BTreeMap::new(),
             indexed: true,
             contains_indexed_global_functions: false,
-            identifiers: Default::default(),
+            identifiers: IdentifierBuilder::new(),
         }
     }
 
@@ -269,7 +251,7 @@ impl Module {
     /// module.set_id("hello");
     /// assert_eq!(module.id(), Some("hello"));
     /// ```
-    #[inline(always)]
+    #[inline]
     #[must_use]
     pub fn id(&self) -> Option<&str> {
         self.id_raw().map(|s| s.as_str())
@@ -327,7 +309,9 @@ impl Module {
     #[inline]
     #[must_use]
     pub fn is_empty(&self) -> bool {
-        self.functions.is_empty()
+        self.indexed
+            && !self.contains_indexed_global_functions
+            && self.functions.is_empty()
             && self.all_functions.is_empty()
             && self.variables.is_empty()
             && self.all_variables.is_empty()
@@ -365,11 +349,10 @@ impl Module {
     /// Generate signatures for all the non-private functions in the [`Module`].
     /// Exported under the `metadata` feature only.
     #[cfg(feature = "metadata")]
-    #[inline(always)]
+    #[inline]
     pub fn gen_fn_signatures(&self) -> impl Iterator<Item = String> + '_ {
-        self.functions
-            .values()
-            .filter(|f| match f.access {
+        self.iter_fn()
+            .filter(|&f| match f.access {
                 FnAccess::Public => true,
                 FnAccess::Private => false,
             })
@@ -402,7 +385,7 @@ impl Module {
     /// module.set_var("answer", 42_i64);
     /// assert_eq!(module.get_var_value::<i64>("answer").expect("answer should exist"), 42);
     /// ```
-    #[inline(always)]
+    #[inline]
     #[must_use]
     pub fn get_var_value<T: Variant + Clone>(&self, name: &str) -> Option<T> {
         self.get_var(name).and_then(Dynamic::try_cast::<T>)
@@ -455,10 +438,11 @@ impl Module {
 
     /// Get a reference to a namespace-qualified variable.
     /// Name and Position in [`EvalAltResult`] are [`None`] and [`NONE`][Position::NONE] and must be set afterwards.
+    #[cfg(not(feature = "no_module"))]
     #[inline]
     pub(crate) fn get_qualified_var(&self, hash_var: u64) -> Result<&Dynamic, Box<EvalAltResult>> {
         self.all_variables.get(&hash_var).ok_or_else(|| {
-            EvalAltResult::ErrorVariableNotFound(String::new(), Position::NONE).into()
+            EvalAltResult::ErrorVariableNotFound(String::new(), crate::Position::NONE).into()
         })
     }
 
@@ -482,7 +466,7 @@ impl Module {
                 namespace: FnNamespace::Internal,
                 access: fn_def.access,
                 params: num_params,
-                param_types: Default::default(),
+                param_types: StaticVec::new_const(),
                 #[cfg(feature = "metadata")]
                 param_names,
                 func: Into::<CallableFunction>::into(fn_def).into(),
@@ -501,13 +485,18 @@ impl Module {
     #[must_use]
     pub fn get_script_fn(
         &self,
-        name: &str,
+        name: impl AsRef<str>,
         num_params: usize,
     ) -> Option<&Shared<crate::ast::ScriptFnDef>> {
-        self.functions
-            .values()
-            .find(|f| f.params == num_params && f.name == name)
-            .and_then(|f| f.func.get_script_fn_def())
+        if self.functions.is_empty() {
+            None
+        } else {
+            let name = name.as_ref();
+
+            self.iter_fn()
+                .find(|f| f.params == num_params && f.name == name)
+                .and_then(|f| f.func.get_script_fn_def())
+        }
     }
 
     /// Get a mutable reference to the underlying [`BTreeMap`] of sub-modules.
@@ -559,7 +548,7 @@ impl Module {
     /// module.set_sub_module("question", sub_module);
     /// assert!(module.get_sub_module("question").is_some());
     /// ```
-    #[inline(always)]
+    #[inline]
     #[must_use]
     pub fn get_sub_module(&self, name: &str) -> Option<&Module> {
         self.modules.get(name).map(|m| m.as_ref())
@@ -623,10 +612,10 @@ impl Module {
     /// In other words, the number of entries should be one larger than the number of parameters.
     #[cfg(feature = "metadata")]
     #[inline]
-    pub fn update_fn_metadata(&mut self, hash_fn: u64, arg_names: &[&str]) -> &mut Self {
+    pub fn update_fn_metadata(&mut self, hash_fn: u64, arg_names: &[impl AsRef<str>]) -> &mut Self {
         let param_names = arg_names
             .iter()
-            .map(|&name| self.identifiers.get(name))
+            .map(|name| self.identifiers.get(name.as_ref()))
             .collect();
 
         if let Some(f) = self.functions.get_mut(&hash_fn) {
@@ -668,7 +657,7 @@ impl Module {
         type_id
     }
 
-    /// Set a Rust function into the [`Module`], returning a hash key.
+    /// Set a Rust function into the [`Module`], returning a non-zero hash key.
     ///
     /// If there is an existing Rust function of the same hash, it is replaced.
     ///
@@ -681,10 +670,11 @@ impl Module {
         name: impl AsRef<str> + Into<Identifier>,
         namespace: FnNamespace,
         access: FnAccess,
-        _arg_names: Option<&[&str]>,
+        arg_names: Option<&[&str]>,
         arg_types: &[TypeId],
         func: CallableFunction,
     ) -> u64 {
+        let _arg_names = arg_names;
         let is_method = func.is_method();
 
         let mut param_types: StaticVec<_> = arg_types
@@ -698,13 +688,13 @@ impl Module {
         #[cfg(feature = "metadata")]
         let mut param_names: StaticVec<_> = _arg_names
             .iter()
-            .flat_map(|p| p.iter())
+            .flat_map(|&p| p.iter())
             .map(|&arg| self.identifiers.get(arg))
             .collect();
         #[cfg(feature = "metadata")]
         param_names.shrink_to_fit();
 
-        let hash_fn = calc_native_fn_hash(empty(), name.as_ref(), &param_types);
+        let hash_fn = calc_native_fn_hash(empty::<&str>(), name.as_ref(), &param_types);
 
         self.functions.insert(
             hash_fn,
@@ -729,7 +719,7 @@ impl Module {
 
     /// Set a Rust function taking a reference to the scripting [`Engine`][crate::Engine],
     /// the current set of functions, plus a list of mutable [`Dynamic`] references
-    /// into the [`Module`], returning a hash key.
+    /// into the [`Module`], returning a non-zero hash key.
     ///
     /// Use this to register a built-in function which must reference settings on the scripting
     /// [`Engine`][crate::Engine] (e.g. to prevent growing an array beyond the allowed maximum size),
@@ -741,7 +731,7 @@ impl Module {
     ///
     /// This function is very low level.
     ///
-    /// # Arguments
+    /// ## Arguments
     ///
     /// A list of [`TypeId`]'s is taken as the argument types.
     ///
@@ -822,7 +812,7 @@ impl Module {
         )
     }
 
-    /// Set a Rust function into the [`Module`], returning a hash key.
+    /// Set a Rust function into the [`Module`], returning a non-zero hash key.
     ///
     /// If there is a similar existing Rust function, it is replaced.
     ///
@@ -861,7 +851,7 @@ impl Module {
         )
     }
 
-    /// Set a Rust getter function taking one mutable parameter, returning a hash key.
+    /// Set a Rust getter function taking one mutable parameter, returning a non-zero hash key.
     /// This function is automatically exposed to the global namespace.
     ///
     /// If there is a similar existing Rust getter function, it is replaced.
@@ -880,7 +870,7 @@ impl Module {
     /// ```
     #[cfg(not(feature = "no_object"))]
     #[inline(always)]
-    pub fn set_getter_fn<ARGS, A, T, F>(&mut self, name: &str, func: F) -> u64
+    pub fn set_getter_fn<ARGS, A, T, F>(&mut self, name: impl AsRef<str>, func: F) -> u64
     where
         A: Variant + Clone,
         T: Variant + Clone,
@@ -898,7 +888,7 @@ impl Module {
     }
 
     /// Set a Rust setter function taking two parameters (the first one mutable) into the [`Module`],
-    /// returning a hash key.
+    /// returning a non-zero hash key.
     /// This function is automatically exposed to the global namespace.
     ///
     /// If there is a similar existing setter Rust function, it is replaced.
@@ -921,7 +911,7 @@ impl Module {
     /// ```
     #[cfg(not(feature = "no_object"))]
     #[inline(always)]
-    pub fn set_setter_fn<ARGS, A, B, F>(&mut self, name: &str, func: F) -> u64
+    pub fn set_setter_fn<ARGS, A, B, F>(&mut self, name: impl AsRef<str>, func: F) -> u64
     where
         A: Variant + Clone,
         B: Variant + Clone,
@@ -939,14 +929,14 @@ impl Module {
     }
 
     /// Set a Rust index getter taking two parameters (the first one mutable) into the [`Module`],
-    /// returning a hash key.
+    /// returning a non-zero hash key.
     /// This function is automatically exposed to the global namespace.
     ///
     /// If there is a similar existing setter Rust function, it is replaced.
     ///
     /// # Panics
     ///
-    /// Panics if the type is [`Array`] or [`Map`].
+    /// Panics if the type is [`Array`][crate::Array] or [`Map`][crate::Map].
     /// Indexers for arrays, object maps and strings cannot be registered.
     ///
     /// # Function Metadata
@@ -975,11 +965,11 @@ impl Module {
         F: Fn(&mut A, B) -> Result<T, Box<EvalAltResult>> + SendSync + 'static,
     {
         #[cfg(not(feature = "no_index"))]
-        if TypeId::of::<A>() == TypeId::of::<Array>() {
+        if TypeId::of::<A>() == TypeId::of::<crate::Array>() {
             panic!("Cannot register indexer for arrays.");
         }
         #[cfg(not(feature = "no_object"))]
-        if TypeId::of::<A>() == TypeId::of::<Map>() {
+        if TypeId::of::<A>() == TypeId::of::<crate::Map>() {
             panic!("Cannot register indexer for object maps.");
         }
         if TypeId::of::<A>() == TypeId::of::<String>()
@@ -1000,14 +990,14 @@ impl Module {
     }
 
     /// Set a Rust index setter taking three parameters (the first one mutable) into the [`Module`],
-    /// returning a hash key.
+    /// returning a non-zero hash key.
     /// This function is automatically exposed to the global namespace.
     ///
     /// If there is a similar existing Rust function, it is replaced.
     ///
     /// # Panics
     ///
-    /// Panics if the type is [`Array`] or [`Map`].
+    /// Panics if the type is [`Array`][crate::Array] or [`Map`][crate::Map].
     /// Indexers for arrays, object maps and strings cannot be registered.
     ///
     /// # Function Metadata
@@ -1036,11 +1026,11 @@ impl Module {
         F: Fn(&mut A, B, C) -> Result<(), Box<EvalAltResult>> + SendSync + 'static,
     {
         #[cfg(not(feature = "no_index"))]
-        if TypeId::of::<A>() == TypeId::of::<Array>() {
+        if TypeId::of::<A>() == TypeId::of::<crate::Array>() {
             panic!("Cannot register indexer for arrays.");
         }
         #[cfg(not(feature = "no_object"))]
-        if TypeId::of::<A>() == TypeId::of::<Map>() {
+        if TypeId::of::<A>() == TypeId::of::<crate::Map>() {
             panic!("Cannot register indexer for object maps.");
         }
         if TypeId::of::<A>() == TypeId::of::<String>()
@@ -1060,7 +1050,7 @@ impl Module {
         )
     }
 
-    /// Set a pair of Rust index getter and setter functions, returning both hash keys.
+    /// Set a pair of Rust index getter and setter functions, returning both non-zero hash keys.
     /// This is a short-hand for [`set_indexer_get_fn`][Module::set_indexer_get_fn] and
     /// [`set_indexer_set_fn`][Module::set_indexer_set_fn].
     ///
@@ -1068,7 +1058,7 @@ impl Module {
     ///
     /// # Panics
     ///
-    /// Panics if the type is [`Array`] or [`Map`].
+    /// Panics if the type is [`Array`][crate::Array] or [`Map`][crate::Map].
     /// Indexers for arrays, object maps and strings cannot be registered.
     ///
     /// # Function Metadata
@@ -1113,7 +1103,7 @@ impl Module {
     /// Get a Rust function.
     ///
     /// The [`u64`] hash is returned by the [`set_native_fn`][Module::set_native_fn] call.
-    #[inline(always)]
+    #[inline]
     #[must_use]
     pub(crate) fn get_fn(&self, hash_fn: u64) -> Option<&CallableFunction> {
         self.functions.get(&hash_fn).map(|f| f.func.as_ref())
@@ -1131,7 +1121,7 @@ impl Module {
     /// Get a namespace-qualified function.
     ///
     /// The [`u64`] hash is calculated by [`build_index`][Module::build_index].
-    #[inline(always)]
+    #[inline]
     #[must_use]
     pub(crate) fn get_qualified_fn(&self, hash_qualified_fn: u64) -> Option<&CallableFunction> {
         self.all_functions
@@ -1152,6 +1142,7 @@ impl Module {
         self.all_type_iterators.clear();
         self.indexed = false;
         self.contains_indexed_global_functions = false;
+        self.identifiers += other.identifiers;
         self
     }
 
@@ -1171,6 +1162,7 @@ impl Module {
         self.all_type_iterators.clear();
         self.indexed = false;
         self.contains_indexed_global_functions = false;
+        self.identifiers += other.identifiers;
         self
     }
 
@@ -1199,6 +1191,7 @@ impl Module {
         self.all_type_iterators.clear();
         self.indexed = false;
         self.contains_indexed_global_functions = false;
+        self.identifiers.merge(&other.identifiers);
         self
     }
 
@@ -1230,7 +1223,7 @@ impl Module {
             other
                 .functions
                 .iter()
-                .filter(|(_, f)| {
+                .filter(|&(_, f)| {
                     _filter(
                         f.namespace,
                         f.access,
@@ -1248,6 +1241,7 @@ impl Module {
         self.all_type_iterators.clear();
         self.indexed = false;
         self.contains_indexed_global_functions = false;
+        self.identifiers.merge(&other.identifiers);
         self
     }
 
@@ -1289,19 +1283,19 @@ impl Module {
     }
 
     /// Get an iterator to the sub-modules in the [`Module`].
-    #[inline(always)]
+    #[inline]
     pub fn iter_sub_modules(&self) -> impl Iterator<Item = (&str, Shared<Module>)> {
         self.modules.iter().map(|(k, m)| (k.as_str(), m.clone()))
     }
 
     /// Get an iterator to the variables in the [`Module`].
-    #[inline(always)]
+    #[inline]
     pub fn iter_var(&self) -> impl Iterator<Item = (&str, &Dynamic)> {
         self.variables.iter().map(|(k, v)| (k.as_str(), v))
     }
 
     /// Get an iterator to the functions in the [`Module`].
-    #[inline(always)]
+    #[inline]
     #[allow(dead_code)]
     pub(crate) fn iter_fn(&self) -> impl Iterator<Item = &FuncInfo> {
         self.functions.values().map(Box::as_ref)
@@ -1316,7 +1310,7 @@ impl Module {
     /// 4) Number of parameters.
     /// 5) Shared reference to function definition [`ScriptFnDef`][crate::ast::ScriptFnDef].
     #[cfg(not(feature = "no_function"))]
-    #[inline(always)]
+    #[inline]
     pub(crate) fn iter_script_fn(
         &self,
     ) -> impl Iterator<
@@ -1328,18 +1322,15 @@ impl Module {
             &Shared<crate::ast::ScriptFnDef>,
         ),
     > + '_ {
-        self.functions
-            .values()
-            .filter(|f| f.func.is_script())
-            .map(|f| {
-                (
-                    f.namespace,
-                    f.access,
-                    f.name.as_str(),
-                    f.params,
-                    f.func.get_script_fn_def().expect("scripted function"),
-                )
-            })
+        self.iter_fn().filter(|&f| f.func.is_script()).map(|f| {
+            (
+                f.namespace,
+                f.access,
+                f.name.as_str(),
+                f.params,
+                f.func.get_script_fn_def().expect("scripted function"),
+            )
+        })
     }
 
     /// Get an iterator over all script-defined functions in the [`Module`].
@@ -1351,13 +1342,12 @@ impl Module {
     /// 4) Number of parameters.
     #[cfg(not(feature = "no_function"))]
     #[cfg(not(feature = "internals"))]
-    #[inline(always)]
+    #[inline]
     pub fn iter_script_fn_info(
         &self,
     ) -> impl Iterator<Item = (FnNamespace, FnAccess, &str, usize)> {
-        self.functions
-            .values()
-            .filter(|f| f.func.is_script())
+        self.iter_fn()
+            .filter(|&f| f.func.is_script())
             .map(|f| (f.namespace, f.access, f.name.as_str(), f.params))
     }
 
@@ -1411,10 +1401,11 @@ impl Module {
     /// ```
     #[cfg(not(feature = "no_module"))]
     pub fn eval_ast_as_new(
-        mut scope: crate::Scope,
+        scope: crate::Scope,
         ast: &crate::AST,
         engine: &crate::Engine,
     ) -> Result<Self, Box<EvalAltResult>> {
+        let mut scope = scope;
         let mut mods = crate::engine::Imports::new();
         let orig_mods_len = mods.len();
 
@@ -1422,21 +1413,28 @@ impl Module {
         engine.eval_ast_with_scope_raw(&mut scope, &mut mods, &ast, 0)?;
 
         // Create new module
-        let mut module = Module::new();
-
-        scope.into_iter().for_each(|(_, value, mut aliases)| {
-            // Variables with an alias left in the scope become module variables
-            match aliases.len() {
-                0 => (),
-                1 => {
-                    let alias = aliases.pop().expect("list has one item");
-                    module.set_var(alias, value);
-                }
-                _ => aliases.into_iter().for_each(|alias| {
-                    module.set_var(alias, value.clone());
-                }),
-            }
-        });
+        let mut module =
+            scope
+                .into_iter()
+                .fold(Module::new(), |mut module, (_, value, mut aliases)| {
+                    // Variables with an alias left in the scope become module variables
+                    match aliases.len() {
+                        0 => (),
+                        1 => {
+                            let alias = aliases.pop().expect("not empty");
+                            module.set_var(alias, value);
+                        }
+                        _ => {
+                            let last_alias = aliases.pop().expect("not empty");
+                            aliases.into_iter().for_each(|alias| {
+                                module.set_var(alias, value.clone());
+                            });
+                            // Avoid cloning the last value
+                            module.set_var(last_alias, value);
+                        }
+                    }
+                    module
+                });
 
         // Extra modules left in the scope become sub-modules
         let mut func_mods = crate::engine::Imports::new();
@@ -1448,26 +1446,27 @@ impl Module {
 
         // Non-private functions defined become module functions
         #[cfg(not(feature = "no_function"))]
-        ast.lib()
-            .functions
-            .values()
-            .filter(|f| match f.access {
-                FnAccess::Public => true,
-                FnAccess::Private => false,
-            })
-            .filter(|f| f.func.is_script())
-            .for_each(|f| {
-                // Encapsulate AST environment
-                let mut func = f
-                    .func
-                    .get_script_fn_def()
-                    .expect("scripted function")
-                    .as_ref()
-                    .clone();
-                func.lib = Some(ast.shared_lib());
-                func.mods = func_mods.clone();
-                module.set_script_fn(func);
-            });
+        if ast.has_functions() {
+            ast.shared_lib()
+                .iter_fn()
+                .filter(|&f| match f.access {
+                    FnAccess::Public => true,
+                    FnAccess::Private => false,
+                })
+                .filter(|&f| f.func.is_script())
+                .for_each(|f| {
+                    // Encapsulate AST environment
+                    let mut func = f
+                        .func
+                        .get_script_fn_def()
+                        .expect("scripted function")
+                        .as_ref()
+                        .clone();
+                    func.lib = Some(ast.shared_lib().clone());
+                    func.mods = func_mods.clone();
+                    module.set_script_fn(func);
+                });
+        }
 
         if let Some(s) = ast.source_raw() {
             module.set_id(s.clone());
@@ -1561,9 +1560,9 @@ impl Module {
 
         if !self.indexed {
             let mut path = Vec::with_capacity(4);
-            let mut variables = Default::default();
-            let mut functions = Default::default();
-            let mut type_iterators = Default::default();
+            let mut variables = BTreeMap::new();
+            let mut functions = BTreeMap::new();
+            let mut type_iterators = BTreeMap::new();
 
             path.push("");
 
@@ -1610,7 +1609,7 @@ impl Module {
     }
 
     /// Set a type iterator into the [`Module`].
-    #[inline(always)]
+    #[inline]
     pub fn set_iterable<T>(&mut self) -> &mut Self
     where
         T: Variant + Clone + IntoIterator,
@@ -1622,7 +1621,7 @@ impl Module {
     }
 
     /// Set an iterator type into the [`Module`] as a type iterator.
-    #[inline(always)]
+    #[inline]
     pub fn set_iterator<T>(&mut self) -> &mut Self
     where
         T: Variant + Clone + Iterator,
@@ -1634,14 +1633,14 @@ impl Module {
     }
 
     /// Get the specified type iterator.
-    #[inline(always)]
+    #[inline]
     #[must_use]
     pub(crate) fn get_qualified_iter(&self, id: TypeId) -> Option<IteratorFn> {
         self.all_type_iterators.get(&id).cloned()
     }
 
     /// Get the specified type iterator.
-    #[inline(always)]
+    #[inline]
     #[must_use]
     pub(crate) fn get_iter(&self, id: TypeId) -> Option<IteratorFn> {
         self.type_iterators.get(&id).cloned()
@@ -1655,10 +1654,6 @@ impl Module {
 ///
 /// A [`StaticVec`] is used because most namespace-qualified access contains only one level,
 /// and it is wasteful to always allocate a [`Vec`] with one element.
-///
-/// # Volatile Data Structure
-///
-/// This type is volatile and may change.
 #[derive(Clone, Eq, PartialEq, Default, Hash)]
 pub struct NamespaceRef {
     index: Option<NonZeroUsize>,
@@ -1677,17 +1672,21 @@ impl fmt::Debug for NamespaceRef {
                 .iter()
                 .map(|Ident { name, .. }| name.as_str())
                 .collect::<StaticVec<_>>()
-                .join("::"),
+                .join(Token::DoubleColon.literal_syntax()),
         )
     }
 }
 
 impl fmt::Display for NamespaceRef {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        for Ident { name, .. } in self.path.iter() {
-            write!(f, "{}{}", name, Token::DoubleColon.syntax())?;
-        }
-        Ok(())
+        f.write_str(
+            &self
+                .path
+                .iter()
+                .map(|Ident { name, .. }| name.as_str())
+                .collect::<StaticVec<_>>()
+                .join(Token::DoubleColon.literal_syntax()),
+        )
     }
 }
 
@@ -1707,10 +1706,20 @@ impl DerefMut for NamespaceRef {
     }
 }
 
+impl From<Vec<Ident>> for NamespaceRef {
+    #[inline(always)]
+    fn from(mut path: Vec<Ident>) -> Self {
+        path.shrink_to_fit();
+        Self {
+            index: None,
+            path: path.into(),
+        }
+    }
+}
+
 impl From<StaticVec<Ident>> for NamespaceRef {
     #[inline(always)]
-    fn from(path: StaticVec<Ident>) -> Self {
-        let mut path = path;
+    fn from(mut path: StaticVec<Ident>) -> Self {
         path.shrink_to_fit();
         Self { index: None, path }
     }
@@ -1720,10 +1729,10 @@ impl NamespaceRef {
     /// Create a new [`NamespaceRef`].
     #[inline(always)]
     #[must_use]
-    pub fn new() -> Self {
+    pub const fn new() -> Self {
         Self {
             index: None,
-            path: StaticVec::new(),
+            path: StaticVec::new_const(),
         }
     }
     /// Get the [`Scope`][crate::Scope] index offset.
