@@ -1,6 +1,9 @@
-use crate::{Engine, EvalAltResult, Identifier, Module, ModuleResolver, Position, Shared};
-#[cfg(feature = "no_std")]
-use std::prelude::v1::*;
+#![cfg(not(feature = "no_std"))]
+#![cfg(not(any(target_arch = "wasm32", target_arch = "wasm64")))]
+
+use crate::func::native::shared_write_lock;
+use crate::{Engine, EvalAltResult, Identifier, Module, ModuleResolver, Position, Scope, Shared};
+
 use std::{
     collections::BTreeMap,
     io::Error as IoError,
@@ -51,13 +54,6 @@ pub struct FileModuleResolver {
     cache: std::sync::RwLock<BTreeMap<PathBuf, Shared<Module>>>,
 }
 
-impl Default for FileModuleResolver {
-    #[inline(always)]
-    fn default() -> Self {
-        Self::new()
-    }
-}
-
 impl FileModuleResolver {
     /// Create a new [`FileModuleResolver`] with the current directory as base path.
     ///
@@ -126,7 +122,7 @@ impl FileModuleResolver {
             base_path: None,
             extension: extension.into(),
             cache_enabled: true,
-            cache: Default::default(),
+            cache: BTreeMap::new().into(),
         }
     }
 
@@ -155,7 +151,7 @@ impl FileModuleResolver {
             base_path: Some(path.into()),
             extension: extension.into(),
             cache_enabled: true,
-            cache: Default::default(),
+            cache: BTreeMap::new().into(),
         }
     }
 
@@ -202,26 +198,19 @@ impl FileModuleResolver {
     /// Is a particular path cached?
     #[inline]
     #[must_use]
-    pub fn is_cached(&self, path: &str, source_path: Option<&str>) -> bool {
+    pub fn is_cached(&self, path: impl AsRef<str>, source_path: Option<&str>) -> bool {
         if !self.cache_enabled {
             return false;
         }
 
-        let file_path = self.get_file_path(path, source_path);
+        let file_path = self.get_file_path(path.as_ref(), source_path);
 
-        #[cfg(not(feature = "sync"))]
-        return self.cache.borrow_mut().contains_key(&file_path);
-        #[cfg(feature = "sync")]
-        return self.cache.write().unwrap().contains_key(&file_path);
+        shared_write_lock(&self.cache).contains_key(&file_path)
     }
     /// Empty the internal cache.
-    #[inline(always)]
+    #[inline]
     pub fn clear_cache(&mut self) -> &mut Self {
-        #[cfg(not(feature = "sync"))]
-        self.cache.borrow_mut().clear();
-        #[cfg(feature = "sync")]
-        self.cache.write().unwrap().clear();
-
+        shared_write_lock(&self.cache).clear();
         self
     }
     /// Remove the specified path from internal cache.
@@ -231,24 +220,14 @@ impl FileModuleResolver {
     #[must_use]
     pub fn clear_cache_for_path(
         &mut self,
-        path: &str,
-        source_path: Option<&str>,
+        path: impl AsRef<str>,
+        source_path: Option<impl AsRef<str>>,
     ) -> Option<Shared<Module>> {
-        let file_path = self.get_file_path(path, source_path);
+        let file_path = self.get_file_path(path.as_ref(), source_path.as_ref().map(|v| v.as_ref()));
 
-        #[cfg(not(feature = "sync"))]
-        return self
-            .cache
-            .borrow_mut()
+        shared_write_lock(&self.cache)
             .remove_entry(&file_path)
-            .map(|(_, v)| v);
-        #[cfg(feature = "sync")]
-        return self
-            .cache
-            .write()
-            .unwrap()
-            .remove_entry(&file_path)
-            .map(|(_, v)| v);
+            .map(|(_, v)| v)
     }
     /// Construct a full file path.
     #[must_use]
@@ -301,7 +280,7 @@ impl ModuleResolver for FileModuleResolver {
         }
 
         // Load the script file and compile it
-        let scope = Default::default();
+        let scope = Scope::new();
 
         let mut ast = engine
             .compile_file(file_path.clone())
@@ -321,10 +300,7 @@ impl ModuleResolver for FileModuleResolver {
 
         // Put it into the cache
         if self.is_cache_enabled() {
-            #[cfg(not(feature = "sync"))]
-            self.cache.borrow_mut().insert(file_path, m.clone());
-            #[cfg(feature = "sync")]
-            self.cache.write().unwrap().insert(file_path, m.clone());
+            shared_write_lock(&self.cache).insert(file_path, m.clone());
         }
 
         Ok(m)
