@@ -9,7 +9,7 @@ use std::prelude::v1::*;
 #[cfg(not(feature = "no_float"))]
 use num_traits::Float;
 
-#[inline(always)]
+#[inline(never)]
 pub fn make_err(msg: impl Into<String>) -> Box<EvalAltResult> {
     EvalAltResult::ErrorArithmetic(msg.into(), Position::NONE).into()
 }
@@ -177,6 +177,8 @@ macro_rules! reg_functions {
 }
 
 def_package!(crate:ArithmeticPackage:"Basic arithmetic", lib, {
+    lib.standard = true;
+
     combine_with_exported_module!(lib, "int", int_functions);
     reg_functions!(lib += signed_basic; INT);
 
@@ -327,13 +329,10 @@ mod f32_functions {
     }
     #[rhai_fn(return_raw)]
     pub fn sign(x: f32) -> Result<INT, Box<EvalAltResult>> {
-        if x == 0.0 {
-            Ok(0)
-        } else {
-            match x.signum() {
-                x if x.is_nan() => Err(make_err("Sign of NaN is undefined")),
-                x => Ok(x as INT),
-            }
+        match x.signum() {
+            _ if x == 0.0 => Ok(0),
+            x if x.is_nan() => Err(make_err("Sign of NaN is undefined")),
+            x => Ok(x as INT),
         }
     }
     pub fn is_zero(x: f32) -> bool {
@@ -437,13 +436,10 @@ mod f64_functions {
     }
     #[rhai_fn(return_raw)]
     pub fn sign(x: f64) -> Result<INT, Box<EvalAltResult>> {
-        if x == 0.0 {
-            Ok(0)
-        } else {
-            match x.signum() {
-                x if x.is_nan() => Err(make_err("Sign of NaN is undefined")),
-                x => Ok(x as INT),
-            }
+        match x.signum() {
+            _ if x == 0.0 => Ok(0),
+            x if x.is_nan() => Err(make_err("Sign of NaN is undefined")),
+            x => Ok(x as INT),
         }
     }
     pub fn is_zero(x: f64) -> bool {
