@@ -9,6 +9,8 @@ use std::prelude::v1::*;
 use std::{any::TypeId, cmp::Ordering, mem};
 
 def_package!(crate:BasicArrayPackage:"Basic array utilities.", lib, {
+    lib.standard = true;
+
     combine_with_exported_module!(lib, "array", array_functions);
 
     // Register array iterator
@@ -67,7 +69,7 @@ mod array_functions {
     }
     #[rhai_fn(return_raw)]
     pub fn pad(
-        _ctx: NativeCallContext,
+        ctx: NativeCallContext,
         array: &mut Array,
         len: INT,
         item: Dynamic,
@@ -76,11 +78,16 @@ mod array_functions {
             return Ok(());
         }
 
+        let _ctx = ctx;
+
         // Check if array will be over max size limit
         #[cfg(not(feature = "unchecked"))]
         if _ctx.engine().max_array_size() > 0 && (len as usize) > _ctx.engine().max_array_size() {
-            return EvalAltResult::ErrorDataTooLarge("Size of array".to_string(), Position::NONE)
-                .into();
+            return Err(EvalAltResult::ErrorDataTooLarge(
+                "Size of array".to_string(),
+                Position::NONE,
+            )
+            .into());
         }
 
         if len as usize > array.len() {
@@ -198,22 +205,7 @@ mod array_functions {
     }
     #[rhai_fn(name = "extract")]
     pub fn extract_tail(array: &mut Array, start: INT) -> Array {
-        if array.is_empty() {
-            return Array::new();
-        }
-
-        let start = if start < 0 {
-            let arr_len = array.len();
-            start
-                .checked_abs()
-                .map_or(0, |n| arr_len - (n as usize).min(arr_len))
-        } else if start as usize >= array.len() {
-            return Array::new();
-        } else {
-            start as usize
-        };
-
-        array[start..].to_vec()
+        extract(array, start, INT::MAX)
     }
     #[rhai_fn(name = "split")]
     pub fn split_at(array: &mut Array, start: INT) -> Array {
@@ -254,12 +246,12 @@ mod array_functions {
         for (i, item) in array.iter().enumerate() {
             ar.push(
                 mapper
-                    .call_dynamic(&ctx, None, [item.clone()])
+                    .call_raw(&ctx, None, [item.clone()])
                     .or_else(|err| match *err {
                         EvalAltResult::ErrorFunctionNotFound(fn_sig, _)
                             if fn_sig.starts_with(mapper.fn_name()) =>
                         {
-                            mapper.call_dynamic(&ctx, None, [item.clone(), (i as INT).into()])
+                            mapper.call_raw(&ctx, None, [item.clone(), (i as INT).into()])
                         }
                         _ => Err(err),
                     })
@@ -276,6 +268,15 @@ mod array_functions {
 
         Ok(ar)
     }
+    #[rhai_fn(name = "map", return_raw, pure)]
+    pub fn map_with_fn_name(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        mapper: &str,
+    ) -> Result<Array, Box<EvalAltResult>> {
+        map(ctx, array, FnPtr::new(mapper)?)
+    }
+
     #[rhai_fn(return_raw, pure)]
     pub fn filter(
         ctx: NativeCallContext,
@@ -290,12 +291,12 @@ mod array_functions {
 
         for (i, item) in array.iter().enumerate() {
             if filter
-                .call_dynamic(&ctx, None, [item.clone()])
+                .call_raw(&ctx, None, [item.clone()])
                 .or_else(|err| match *err {
                     EvalAltResult::ErrorFunctionNotFound(fn_sig, _)
                         if fn_sig.starts_with(filter.fn_name()) =>
                     {
-                        filter.call_dynamic(&ctx, None, [item.clone(), (i as INT).into()])
+                        filter.call_raw(&ctx, None, [item.clone(), (i as INT).into()])
                     }
                     _ => Err(err),
                 })
@@ -316,6 +317,14 @@ mod array_functions {
 
         Ok(ar)
     }
+    #[rhai_fn(name = "filter", return_raw, pure)]
+    pub fn filter_with_fn_name(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        filter_func: &str,
+    ) -> Result<Array, Box<EvalAltResult>> {
+        filter(ctx, array, FnPtr::new(filter_func)?)
+    }
     #[rhai_fn(return_raw, pure)]
     pub fn contains(
         ctx: NativeCallContext,
@@ -328,7 +337,7 @@ mod array_functions {
 
         for item in array.iter_mut() {
             if ctx
-                .call_fn_dynamic_raw(OP_EQUALS, true, &mut [item, &mut value.clone()])
+                .call_fn_raw(OP_EQUALS, true, false, &mut [item, &mut value.clone()])
                 .or_else(|err| match *err {
                     EvalAltResult::ErrorFunctionNotFound(ref fn_sig, _)
                         if fn_sig.starts_with(OP_EQUALS) =>
@@ -387,7 +396,7 @@ mod array_functions {
 
         for (i, item) in array.iter_mut().enumerate().skip(start) {
             if ctx
-                .call_fn_dynamic_raw(OP_EQUALS, true, &mut [item, &mut value.clone()])
+                .call_fn_raw(OP_EQUALS, true, false, &mut [item, &mut value.clone()])
                 .or_else(|err| match *err {
                     EvalAltResult::ErrorFunctionNotFound(ref fn_sig, _)
                         if fn_sig.starts_with(OP_EQUALS) =>
@@ -411,6 +420,14 @@ mod array_functions {
         Ok(-1 as INT)
     }
     #[rhai_fn(name = "index_of", return_raw, pure)]
+    pub fn index_of_with_fn_name(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        filter: &str,
+    ) -> Result<INT, Box<EvalAltResult>> {
+        index_of_filter(ctx, array, FnPtr::new(filter)?)
+    }
+    #[rhai_fn(name = "index_of", return_raw, pure)]
     pub fn index_of_filter(
         ctx: NativeCallContext,
         array: &mut Array,
@@ -446,12 +463,12 @@ mod array_functions {
 
         for (i, item) in array.iter().enumerate().skip(start) {
             if filter
-                .call_dynamic(&ctx, None, [item.clone()])
+                .call_raw(&ctx, None, [item.clone()])
                 .or_else(|err| match *err {
                     EvalAltResult::ErrorFunctionNotFound(fn_sig, _)
                         if fn_sig.starts_with(filter.fn_name()) =>
                     {
-                        filter.call_dynamic(&ctx, None, [item.clone(), (i as INT).into()])
+                        filter.call_raw(&ctx, None, [item.clone(), (i as INT).into()])
                     }
                     _ => Err(err),
                 })
@@ -472,6 +489,15 @@ mod array_functions {
 
         Ok(-1 as INT)
     }
+    #[rhai_fn(name = "index_of", return_raw, pure)]
+    pub fn index_of_with_fn_name_filter_starting_from(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        filter: &str,
+        start: INT,
+    ) -> Result<INT, Box<EvalAltResult>> {
+        index_of_filter_starting_from(ctx, array, FnPtr::new(filter)?, start)
+    }
     #[rhai_fn(return_raw, pure)]
     pub fn some(
         ctx: NativeCallContext,
@@ -484,12 +510,12 @@ mod array_functions {
 
         for (i, item) in array.iter().enumerate() {
             if filter
-                .call_dynamic(&ctx, None, [item.clone()])
+                .call_raw(&ctx, None, [item.clone()])
                 .or_else(|err| match *err {
                     EvalAltResult::ErrorFunctionNotFound(fn_sig, _)
                         if fn_sig.starts_with(filter.fn_name()) =>
                     {
-                        filter.call_dynamic(&ctx, None, [item.clone(), (i as INT).into()])
+                        filter.call_raw(&ctx, None, [item.clone(), (i as INT).into()])
                     }
                     _ => Err(err),
                 })
@@ -510,6 +536,14 @@ mod array_functions {
 
         Ok(false)
     }
+    #[rhai_fn(name = "some", return_raw, pure)]
+    pub fn some_with_fn_name(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        filter: &str,
+    ) -> Result<bool, Box<EvalAltResult>> {
+        some(ctx, array, FnPtr::new(filter)?)
+    }
     #[rhai_fn(return_raw, pure)]
     pub fn all(
         ctx: NativeCallContext,
@@ -522,12 +556,12 @@ mod array_functions {
 
         for (i, item) in array.iter().enumerate() {
             if !filter
-                .call_dynamic(&ctx, None, [item.clone()])
+                .call_raw(&ctx, None, [item.clone()])
                 .or_else(|err| match *err {
                     EvalAltResult::ErrorFunctionNotFound(fn_sig, _)
                         if fn_sig.starts_with(filter.fn_name()) =>
                     {
-                        filter.call_dynamic(&ctx, None, [item.clone(), (i as INT).into()])
+                        filter.call_raw(&ctx, None, [item.clone(), (i as INT).into()])
                     }
                     _ => Err(err),
                 })
@@ -548,26 +582,85 @@ mod array_functions {
 
         Ok(true)
     }
+    #[rhai_fn(name = "all", return_raw, pure)]
+    pub fn all_with_fn_name(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        filter: &str,
+    ) -> Result<bool, Box<EvalAltResult>> {
+        all(ctx, array, FnPtr::new(filter)?)
+    }
+    #[rhai_fn(return_raw)]
+    pub fn dedup(ctx: NativeCallContext, array: &mut Array) -> Result<(), Box<EvalAltResult>> {
+        dedup_with_fn_name(ctx, array, OP_EQUALS)
+    }
+    #[rhai_fn(name = "dedup", return_raw)]
+    pub fn dedup_by_comparer(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        comparer: FnPtr,
+    ) -> Result<(), Box<EvalAltResult>> {
+        if array.is_empty() {
+            return Ok(());
+        }
+
+        array.dedup_by(|x, y| {
+            comparer
+                .call_raw(&ctx, None, [x.clone(), y.clone()])
+                .unwrap_or_else(|_| Dynamic::FALSE)
+                .as_bool()
+                .unwrap_or(false)
+        });
+
+        Ok(())
+    }
+    #[rhai_fn(name = "dedup", return_raw)]
+    fn dedup_with_fn_name(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        comparer: &str,
+    ) -> Result<(), Box<EvalAltResult>> {
+        dedup_by_comparer(ctx, array, FnPtr::new(comparer)?)
+    }
     #[rhai_fn(return_raw, pure)]
     pub fn reduce(
         ctx: NativeCallContext,
         array: &mut Array,
         reducer: FnPtr,
+    ) -> Result<Dynamic, Box<EvalAltResult>> {
+        reduce_with_initial(ctx, array, reducer, Dynamic::UNIT)
+    }
+    #[rhai_fn(name = "reduce", return_raw, pure)]
+    pub fn reduce_with_fn_name(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        reducer: &str,
+    ) -> Result<Dynamic, Box<EvalAltResult>> {
+        reduce(ctx, array, FnPtr::new(reducer)?)
+    }
+    #[rhai_fn(name = "reduce", return_raw, pure)]
+    pub fn reduce_with_initial(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        reducer: FnPtr,
+        initial: Dynamic,
     ) -> Result<Dynamic, Box<EvalAltResult>> {
         if array.is_empty() {
-            return Ok(Dynamic::UNIT);
+            return Ok(initial);
         }
 
-        let mut result = Dynamic::UNIT;
+        let mut result = initial;
 
         for (i, item) in array.iter().enumerate() {
+            let item = item.clone();
+
             result = reducer
-                .call_dynamic(&ctx, None, [result.clone(), item.clone()])
+                .call_raw(&ctx, None, [result.clone(), item.clone()])
                 .or_else(|err| match *err {
                     EvalAltResult::ErrorFunctionNotFound(fn_sig, _)
                         if fn_sig.starts_with(reducer.fn_name()) =>
                     {
-                        reducer.call_dynamic(&ctx, None, [result, item.clone(), (i as INT).into()])
+                        reducer.call_raw(&ctx, None, [result, item, (i as INT).into()])
                     }
                     _ => Err(err),
                 })
@@ -584,7 +677,32 @@ mod array_functions {
         Ok(result)
     }
     #[rhai_fn(name = "reduce", return_raw, pure)]
-    pub fn reduce_with_initial(
+    pub fn reduce_with_fn_name_with_initial(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        reducer: &str,
+        initial: Dynamic,
+    ) -> Result<Dynamic, Box<EvalAltResult>> {
+        reduce_with_initial(ctx, array, FnPtr::new(reducer)?, initial)
+    }
+    #[rhai_fn(return_raw, pure)]
+    pub fn reduce_rev(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        reducer: FnPtr,
+    ) -> Result<Dynamic, Box<EvalAltResult>> {
+        reduce_rev_with_initial(ctx, array, reducer, Dynamic::UNIT)
+    }
+    #[rhai_fn(name = "reduce_rev", return_raw, pure)]
+    pub fn reduce_rev_with_fn_name(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        reducer: &str,
+    ) -> Result<Dynamic, Box<EvalAltResult>> {
+        reduce_rev(ctx, array, FnPtr::new(reducer)?)
+    }
+    #[rhai_fn(name = "reduce_rev", return_raw, pure)]
+    pub fn reduce_rev_with_initial(
         ctx: NativeCallContext,
         array: &mut Array,
         reducer: FnPtr,
@@ -595,50 +713,18 @@ mod array_functions {
         }
 
         let mut result = initial;
+        let len = array.len();
+
+        for (i, item) in array.iter().rev().enumerate() {
+            let item = item.clone();
 
-        for (i, item) in array.iter().enumerate() {
             result = reducer
-                .call_dynamic(&ctx, None, [result.clone(), item.clone()])
+                .call_raw(&ctx, None, [result.clone(), item.clone()])
                 .or_else(|err| match *err {
                     EvalAltResult::ErrorFunctionNotFound(fn_sig, _)
                         if fn_sig.starts_with(reducer.fn_name()) =>
                     {
-                        reducer.call_dynamic(&ctx, None, [result, item.clone(), (i as INT).into()])
-                    }
-                    _ => Err(err),
-                })
-                .map_err(|err| {
-                    Box::new(EvalAltResult::ErrorInFunctionCall(
-                        "reduce".to_string(),
-                        ctx.source().unwrap_or("").to_string(),
-                        err,
-                        Position::NONE,
-                    ))
-                })?;
-        }
-
-        Ok(result)
-    }
-    #[rhai_fn(return_raw, pure)]
-    pub fn reduce_rev(
-        ctx: NativeCallContext,
-        array: &mut Array,
-        reducer: FnPtr,
-    ) -> Result<Dynamic, Box<EvalAltResult>> {
-        if array.is_empty() {
-            return Ok(Dynamic::UNIT);
-        }
-
-        let mut result = Dynamic::UNIT;
-
-        for (i, item) in array.iter().enumerate().rev() {
-            result = reducer
-                .call_dynamic(&ctx, None, [result.clone(), item.clone()])
-                .or_else(|err| match *err {
-                    EvalAltResult::ErrorFunctionNotFound(fn_sig, _)
-                        if fn_sig.starts_with(reducer.fn_name()) =>
-                    {
-                        reducer.call_dynamic(&ctx, None, [result, item.clone(), (i as INT).into()])
+                        reducer.call_raw(&ctx, None, [result, item, ((len - 1 - i) as INT).into()])
                     }
                     _ => Err(err),
                 })
@@ -655,40 +741,21 @@ mod array_functions {
         Ok(result)
     }
     #[rhai_fn(name = "reduce_rev", return_raw, pure)]
-    pub fn reduce_rev_with_initial(
+    pub fn reduce_rev_with_fn_name_with_initial(
         ctx: NativeCallContext,
         array: &mut Array,
-        reducer: FnPtr,
+        reducer: &str,
         initial: Dynamic,
     ) -> Result<Dynamic, Box<EvalAltResult>> {
-        if array.is_empty() {
-            return Ok(initial);
-        }
-
-        let mut result = initial;
-
-        for (i, item) in array.iter().enumerate().rev() {
-            result = reducer
-                .call_dynamic(&ctx, None, [result.clone(), item.clone()])
-                .or_else(|err| match *err {
-                    EvalAltResult::ErrorFunctionNotFound(fn_sig, _)
-                        if fn_sig.starts_with(reducer.fn_name()) =>
-                    {
-                        reducer.call_dynamic(&ctx, None, [result, item.clone(), (i as INT).into()])
-                    }
-                    _ => Err(err),
-                })
-                .map_err(|err| {
-                    Box::new(EvalAltResult::ErrorInFunctionCall(
-                        "reduce_rev".to_string(),
-                        ctx.source().unwrap_or("").to_string(),
-                        err,
-                        Position::NONE,
-                    ))
-                })?;
-        }
-
-        Ok(result)
+        reduce_rev_with_initial(ctx, array, FnPtr::new(reducer)?, initial)
+    }
+    #[rhai_fn(name = "sort", return_raw)]
+    pub fn sort_with_fn_name(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        comparer: &str,
+    ) -> Result<(), Box<EvalAltResult>> {
+        sort(ctx, array, FnPtr::new(comparer)?)
     }
     #[rhai_fn(return_raw)]
     pub fn sort(
@@ -696,13 +763,13 @@ mod array_functions {
         array: &mut Array,
         comparer: FnPtr,
     ) -> Result<(), Box<EvalAltResult>> {
-        if array.is_empty() {
+        if array.len() <= 1 {
             return Ok(());
         }
 
         array.sort_by(|x, y| {
             comparer
-                .call_dynamic(&ctx, None, [x.clone(), y.clone()])
+                .call_raw(&ctx, None, [x.clone(), y.clone()])
                 .ok()
                 .and_then(|v| v.as_int().ok())
                 .map(|v| match v {
@@ -716,6 +783,78 @@ mod array_functions {
 
         Ok(())
     }
+    #[rhai_fn(name = "sort", return_raw)]
+    pub fn sort_with_builtin(array: &mut Array) -> Result<(), Box<EvalAltResult>> {
+        if array.len() <= 1 {
+            return Ok(());
+        }
+
+        let type_id = array[0].type_id();
+
+        if array.iter().any(|a| a.type_id() != type_id) {
+            return Err(EvalAltResult::ErrorFunctionNotFound(
+                "sort() cannot be called with elements of different types".into(),
+                Position::NONE,
+            )
+            .into());
+        }
+
+        if type_id == TypeId::of::<INT>() {
+            array.sort_by(|a, b| {
+                let a = a.as_int().expect("`INT`");
+                let b = b.as_int().expect("`INT`");
+                a.cmp(&b)
+            });
+            return Ok(());
+        }
+        if type_id == TypeId::of::<char>() {
+            array.sort_by(|a, b| {
+                let a = a.as_char().expect("char");
+                let b = b.as_char().expect("char");
+                a.cmp(&b)
+            });
+            return Ok(());
+        }
+        #[cfg(not(feature = "no_float"))]
+        if type_id == TypeId::of::<crate::FLOAT>() {
+            array.sort_by(|a, b| {
+                let a = a.as_float().expect("`FLOAT`");
+                let b = b.as_float().expect("`FLOAT`");
+                a.partial_cmp(&b).unwrap_or(Ordering::Equal)
+            });
+            return Ok(());
+        }
+        if type_id == TypeId::of::<ImmutableString>() {
+            array.sort_by(|a, b| {
+                let a = a.read_lock::<ImmutableString>().expect("`ImmutableString`");
+                let b = b.read_lock::<ImmutableString>().expect("`ImmutableString`");
+                a.as_str().cmp(b.as_str())
+            });
+            return Ok(());
+        }
+        #[cfg(feature = "decimal")]
+        if type_id == TypeId::of::<rust_decimal::Decimal>() {
+            array.sort_by(|a, b| {
+                let a = a.as_decimal().expect("`Decimal`");
+                let b = b.as_decimal().expect("`Decimal`");
+                a.cmp(&b)
+            });
+            return Ok(());
+        }
+        if type_id == TypeId::of::<bool>() {
+            array.sort_by(|a, b| {
+                let a = a.as_bool().expect("`bool`");
+                let b = b.as_bool().expect("`bool`");
+                a.cmp(&b)
+            });
+            return Ok(());
+        }
+        if type_id == TypeId::of::<()>() {
+            return Ok(());
+        }
+
+        Ok(())
+    }
     #[rhai_fn(return_raw)]
     pub fn drain(
         ctx: NativeCallContext,
@@ -733,12 +872,12 @@ mod array_functions {
 
         while x < array.len() {
             if filter
-                .call_dynamic(&ctx, None, [array[x].clone()])
+                .call_raw(&ctx, None, [array[x].clone()])
                 .or_else(|err| match *err {
                     EvalAltResult::ErrorFunctionNotFound(fn_sig, _)
                         if fn_sig.starts_with(filter.fn_name()) =>
                     {
-                        filter.call_dynamic(&ctx, None, [array[x].clone(), (i as INT).into()])
+                        filter.call_raw(&ctx, None, [array[x].clone(), (i as INT).into()])
                     }
                     _ => Err(err),
                 })
@@ -763,6 +902,14 @@ mod array_functions {
 
         Ok(drained)
     }
+    #[rhai_fn(name = "drain", return_raw)]
+    pub fn drain_with_fn_name(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        filter: &str,
+    ) -> Result<Array, Box<EvalAltResult>> {
+        drain(ctx, array, FnPtr::new(filter)?)
+    }
     #[rhai_fn(name = "drain")]
     pub fn drain_range(array: &mut Array, start: INT, len: INT) -> Array {
         if array.is_empty() || len <= 0 {
@@ -807,12 +954,12 @@ mod array_functions {
 
         while x < array.len() {
             if !filter
-                .call_dynamic(&ctx, None, [array[x].clone()])
+                .call_raw(&ctx, None, [array[x].clone()])
                 .or_else(|err| match *err {
                     EvalAltResult::ErrorFunctionNotFound(fn_sig, _)
                         if fn_sig.starts_with(filter.fn_name()) =>
                     {
-                        filter.call_dynamic(&ctx, None, [array[x].clone(), (i as INT).into()])
+                        filter.call_raw(&ctx, None, [array[x].clone(), (i as INT).into()])
                     }
                     _ => Err(err),
                 })
@@ -837,6 +984,14 @@ mod array_functions {
 
         Ok(drained)
     }
+    #[rhai_fn(name = "retain", return_raw)]
+    pub fn retain_with_fn_name(
+        ctx: NativeCallContext,
+        array: &mut Array,
+        filter: &str,
+    ) -> Result<Array, Box<EvalAltResult>> {
+        retain(ctx, array, FnPtr::new(filter)?)
+    }
     #[rhai_fn(name = "retain")]
     pub fn retain_range(array: &mut Array, start: INT, len: INT) -> Array {
         if array.is_empty() || len <= 0 {
@@ -884,7 +1039,7 @@ mod array_functions {
 
         for (a1, a2) in array1.iter_mut().zip(array2.iter_mut()) {
             if !ctx
-                .call_fn_dynamic_raw(OP_EQUALS, true, &mut [a1, a2])
+                .call_fn_raw(OP_EQUALS, true, false, &mut [a1, a2])
                 .or_else(|err| match *err {
                     EvalAltResult::ErrorFunctionNotFound(ref fn_sig, _)
                         if fn_sig.starts_with(OP_EQUALS) =>
