@@ -4,6 +4,8 @@ use crate::{def_package, FnPtr, ImmutableString, NativeCallContext};
 use std::prelude::v1::*;
 
 def_package!(crate:BasicFnPackage:"Basic Fn functions.", lib, {
+    lib.standard = true;
+
     combine_with_exported_module!(lib, "FnPtr", fn_ptr_functions);
 });
 
@@ -15,20 +17,16 @@ mod fn_ptr_functions {
     }
 
     #[cfg(not(feature = "no_function"))]
-    pub mod functions {
-        #[rhai_fn(name = "is_anonymous", get = "is_anonymous", pure)]
-        pub fn is_anonymous(fn_ptr: &mut FnPtr) -> bool {
-            fn_ptr.is_anonymous()
-        }
+    #[rhai_fn(name = "is_anonymous", get = "is_anonymous", pure)]
+    pub fn is_anonymous(fn_ptr: &mut FnPtr) -> bool {
+        fn_ptr.is_anonymous()
     }
 
     #[cfg(not(feature = "no_function"))]
     #[cfg(not(feature = "no_index"))]
     #[cfg(not(feature = "no_object"))]
-    pub mod functions_and_maps {
-        pub fn get_fn_metadata_list(ctx: NativeCallContext) -> crate::Array {
-            collect_fn_metadata(ctx)
-        }
+    pub fn get_fn_metadata_list(ctx: NativeCallContext) -> crate::Array {
+        collect_fn_metadata(ctx)
     }
 }
 
@@ -92,11 +90,13 @@ fn collect_fn_metadata(ctx: NativeCallContext) -> crate::Array {
     .map(|&s| s.into())
     .collect();
 
-    let mut list = Array::new();
-
-    ctx.iter_namespaces()
-        .flat_map(|m| m.iter_script_fn())
-        .for_each(|(_, _, _, _, f)| list.push(make_metadata(&dict, None, f).into()));
+    let mut _list = ctx.iter_namespaces().flat_map(Module::iter_script_fn).fold(
+        Array::new(),
+        |mut list, (_, _, _, _, f)| {
+            list.push(make_metadata(&dict, None, f).into());
+            list
+        },
+    );
 
     #[cfg(not(feature = "no_module"))]
     {
@@ -111,14 +111,19 @@ fn collect_fn_metadata(ctx: NativeCallContext) -> crate::Array {
                 list.push(make_metadata(dict, Some(namespace.clone()), f).into())
             });
             module.iter_sub_modules().for_each(|(ns, m)| {
-                let ns = format!("{}::{}", namespace, ns);
+                let ns = format!(
+                    "{}{}{}",
+                    namespace,
+                    crate::tokenizer::Token::DoubleColon.literal_syntax(),
+                    ns
+                );
                 scan_module(list, dict, ns.into(), m.as_ref())
             });
         }
 
         ctx.iter_imports_raw()
-            .for_each(|(ns, m)| scan_module(&mut list, &dict, ns.clone(), m.as_ref()));
+            .for_each(|(ns, m)| scan_module(&mut _list, &dict, ns.clone(), m.as_ref()));
     }
 
-    list
+    _list
 }
