@@ -1,4 +1,4 @@
-use crate::dynamic::Variant;
+use crate::types::dynamic::Variant;
 use crate::{def_package, EvalAltResult, INT};
 use std::iter::{ExactSizeIterator, FusedIterator};
 use std::ops::Range;
@@ -25,9 +25,9 @@ where
         #[cfg(not(feature = "unchecked"))]
         if let Some(r) = from.checked_add(&step) {
             if r == from {
-                return EvalAltResult::ErrorInFunctionCall(
+                return Err(EvalAltResult::ErrorInFunctionCall(
                     "range".to_string(),
-                    Default::default(),
+                    String::new(),
                     EvalAltResult::ErrorArithmetic(
                         "step value cannot be zero".to_string(),
                         crate::Position::NONE,
@@ -35,7 +35,7 @@ where
                     .into(),
                     crate::Position::NONE,
                 )
-                .into();
+                .into());
             }
         }
 
@@ -122,21 +122,27 @@ impl BitRange {
 
             #[cfg(not(feature = "unchecked"))]
             if offset >= BITS {
-                return EvalAltResult::ErrorBitFieldBounds(BITS, from, crate::Position::NONE)
-                    .into();
+                return Err(
+                    EvalAltResult::ErrorBitFieldBounds(BITS, from, crate::Position::NONE).into(),
+                );
             }
             offset
         } else {
             #[cfg(not(feature = "unchecked"))]
             if let Some(abs_from) = from.checked_abs() {
                 if (abs_from as usize) > BITS {
-                    return EvalAltResult::ErrorBitFieldBounds(BITS, from, crate::Position::NONE)
-                        .into();
+                    return Err(EvalAltResult::ErrorBitFieldBounds(
+                        BITS,
+                        from,
+                        crate::Position::NONE,
+                    )
+                    .into());
                 }
                 BITS - (abs_from as usize)
             } else {
-                return EvalAltResult::ErrorBitFieldBounds(BITS, from, crate::Position::NONE)
-                    .into();
+                return Err(
+                    EvalAltResult::ErrorBitFieldBounds(BITS, from, crate::Position::NONE).into(),
+                );
             }
 
             #[cfg(feature = "unchecked")]
@@ -195,7 +201,7 @@ struct CharsStream(Vec<char>, usize);
 impl CharsStream {
     pub fn new(string: &str, from: INT, len: INT) -> Self {
         if len <= 0 {
-            return Self(Default::default(), 0);
+            return Self(Vec::new(), 0);
         }
         if from >= 0 {
             return Self(
@@ -245,7 +251,7 @@ impl Iterator for CharsStream {
         }
     }
 
-    #[inline(always)]
+    #[inline]
     fn size_hint(&self) -> (usize, Option<usize>) {
         let remaining = self.0.len() - self.1;
         (remaining, Some(remaining))
@@ -255,7 +261,7 @@ impl Iterator for CharsStream {
 impl FusedIterator for CharsStream {}
 
 impl ExactSizeIterator for CharsStream {
-    #[inline(always)]
+    #[inline]
     fn len(&self) -> usize {
         self.0.len() - self.1
     }
@@ -292,6 +298,8 @@ macro_rules! reg_range {
 }
 
 def_package!(crate:BasicIteratorPackage:"Basic range iterators.", lib, {
+    lib.standard = true;
+
     reg_range!(lib | "range" => INT);
 
     #[cfg(not(feature = "only_i32"))]
@@ -325,10 +333,10 @@ def_package!(crate:BasicIteratorPackage:"Basic range iterators.", lib, {
             pub fn new(from: FLOAT, to: FLOAT, step: FLOAT) -> Result<Self, Box<EvalAltResult>> {
                 #[cfg(not(feature = "unchecked"))]
                 if step == 0.0 {
-                    return EvalAltResult::ErrorInFunctionCall("range".to_string(), "".to_string(),
+                    return Err(EvalAltResult::ErrorInFunctionCall("range".to_string(), "".to_string(),
                         EvalAltResult::ErrorArithmetic("step value cannot be zero".to_string(), crate::Position::NONE).into(),
                         crate::Position::NONE,
-                    ).into();
+                    ).into());
                 }
 
                 Ok(Self(from, to, step))
@@ -387,10 +395,10 @@ def_package!(crate:BasicIteratorPackage:"Basic range iterators.", lib, {
             pub fn new(from: Decimal, to: Decimal, step: Decimal) -> Result<Self, Box<EvalAltResult>> {
                 #[cfg(not(feature = "unchecked"))]
                 if step.is_zero() {
-                    return EvalAltResult::ErrorInFunctionCall("range".to_string(), "".to_string(),
+                    return Err(EvalAltResult::ErrorInFunctionCall("range".to_string(), "".to_string(),
                         EvalAltResult::ErrorArithmetic("step value cannot be zero".to_string(), crate::Position::NONE).into(),
                         crate::Position::NONE,
-                    ).into();
+                    ).into());
                 }
 
                 Ok(Self(from, to, step))
