@@ -38,6 +38,8 @@ macro_rules! reg_functions {
 }
 
 def_package!(crate:LogicPackage:"Logical operators.", lib, {
+    lib.standard = true;
+
     #[cfg(not(feature = "only_i32"))]
     #[cfg(not(feature = "only_i64"))]
     {
@@ -202,7 +204,7 @@ mod bit_field_functions {
             let offset = index as usize;
 
             if offset >= BITS {
-                EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into()
+                Err(EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into())
             } else {
                 Ok((value & (1 << offset)) != 0)
             }
@@ -211,12 +213,12 @@ mod bit_field_functions {
 
             // Count from end if negative
             if offset > BITS {
-                EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into()
+                Err(EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into())
             } else {
                 Ok((value & (1 << (BITS - offset))) != 0)
             }
         } else {
-            EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into()
+            Err(EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into())
         }
     }
     #[rhai_fn(return_raw)]
@@ -225,7 +227,7 @@ mod bit_field_functions {
             let offset = index as usize;
 
             if offset >= BITS {
-                EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into()
+                Err(EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into())
             } else {
                 let mask = 1 << offset;
                 if new_value {
@@ -240,7 +242,7 @@ mod bit_field_functions {
 
             // Count from end if negative
             if offset > BITS {
-                EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into()
+                Err(EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into())
             } else {
                 let mask = 1 << offset;
                 if new_value {
@@ -251,7 +253,7 @@ mod bit_field_functions {
                 Ok(())
             }
         } else {
-            EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into()
+            Err(EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into())
         }
     }
     #[rhai_fn(return_raw)]
@@ -264,7 +266,7 @@ mod bit_field_functions {
             let offset = index as usize;
 
             if offset >= BITS {
-                return EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into();
+                return Err(EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into());
             }
 
             offset
@@ -273,11 +275,11 @@ mod bit_field_functions {
 
             // Count from end if negative
             if offset > BITS {
-                return EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into();
+                return Err(EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into());
             }
             BITS - offset
         } else {
-            return EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into();
+            return Err(EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into());
         };
 
         let bits = if offset + bits as usize > BITS {
@@ -311,7 +313,7 @@ mod bit_field_functions {
             let offset = index as usize;
 
             if offset >= BITS {
-                return EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into();
+                return Err(EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into());
             }
 
             offset
@@ -320,11 +322,11 @@ mod bit_field_functions {
 
             // Count from end if negative
             if offset > BITS {
-                return EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into();
+                return Err(EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into());
             }
             BITS - offset
         } else {
-            return EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into();
+            return Err(EvalAltResult::ErrorBitFieldBounds(BITS, index, Position::NONE).into());
         };
 
         let bits = if offset + bits as usize > BITS {
