@@ -8,9 +8,6 @@ use std::prelude::v1::*;
 #[cfg(not(feature = "no_float"))]
 use crate::FLOAT;
 
-#[cfg(not(feature = "no_float"))]
-use crate::error::EvalAltResult;
-
 #[cfg(feature = "no_std")]
 #[cfg(not(feature = "no_float"))]
 use num_traits::Float;
@@ -62,6 +59,8 @@ macro_rules! reg_functions {
 }
 
 def_package!(crate:BasicMathPackage:"Basic mathematic functions.", lib, {
+    lib.standard = true;
+
     // Integer functions
     combine_with_exported_module!(lib, "int", int_functions);
 
@@ -114,11 +113,11 @@ mod int_functions {
     #[rhai_fn(name = "parse_int", return_raw)]
     pub fn parse_int_radix(string: &str, radix: INT) -> Result<INT, Box<EvalAltResult>> {
         if !(2..=36).contains(&radix) {
-            return EvalAltResult::ErrorArithmetic(
+            return Err(EvalAltResult::ErrorArithmetic(
                 format!("Invalid radix: '{}'", radix),
                 Position::NONE,
             )
-            .into();
+            .into());
         }
 
         INT::from_str_radix(string.trim(), radix as u32).map_err(|err| {
@@ -261,11 +260,11 @@ mod float_functions {
     #[rhai_fn(name = "to_int", return_raw)]
     pub fn f32_to_int(x: f32) -> Result<INT, Box<EvalAltResult>> {
         if cfg!(not(feature = "unchecked")) && x > (MAX_INT as f32) {
-            EvalAltResult::ErrorArithmetic(
+            Err(EvalAltResult::ErrorArithmetic(
                 format!("Integer overflow: to_int({})", x),
                 Position::NONE,
             )
-            .into()
+            .into())
         } else {
             Ok(x.trunc() as INT)
         }
@@ -273,11 +272,11 @@ mod float_functions {
     #[rhai_fn(name = "to_int", return_raw)]
     pub fn f64_to_int(x: f64) -> Result<INT, Box<EvalAltResult>> {
         if cfg!(not(feature = "unchecked")) && x > (MAX_INT as f64) {
-            EvalAltResult::ErrorArithmetic(
+            Err(EvalAltResult::ErrorArithmetic(
                 format!("Integer overflow: to_int({})", x),
                 Position::NONE,
             )
-            .into()
+            .into())
         } else {
             Ok(x.trunc() as INT)
         }
@@ -293,11 +292,9 @@ mod float_functions {
         })
     }
     #[cfg(not(feature = "f32_float"))]
-    pub mod f32_f64 {
-        #[rhai_fn(name = "to_float")]
-        pub fn f32_to_f64(x: f32) -> f64 {
-            x as f64
-        }
+    #[rhai_fn(name = "to_float")]
+    pub fn f32_to_f64(x: f32) -> f64 {
+        x as f64
     }
 }
 
@@ -308,21 +305,23 @@ mod decimal_functions {
         prelude::{FromStr, RoundingStrategy},
         Decimal, MathematicalOps,
     };
+    #[cfg(not(feature = "no_float"))]
+    use std::convert::TryFrom;
 
     #[cfg(feature = "no_float")]
-    pub mod float_polyfills {
-        #[rhai_fn(name = "PI")]
-        pub fn pi() -> Decimal {
-            Decimal::PI
-        }
-        #[rhai_fn(name = "E")]
-        pub fn e() -> Decimal {
-            Decimal::E
-        }
-        #[rhai_fn(return_raw)]
-        pub fn parse_float(s: &str) -> Result<Decimal, Box<EvalAltResult>> {
-            super::parse_decimal(s)
-        }
+    #[rhai_fn(name = "PI")]
+    pub fn pi() -> Decimal {
+        Decimal::PI
+    }
+    #[cfg(feature = "no_float")]
+    #[rhai_fn(name = "E")]
+    pub fn e() -> Decimal {
+        Decimal::E
+    }
+    #[cfg(feature = "no_float")]
+    #[rhai_fn(return_raw)]
+    pub fn parse_float(s: &str) -> Result<Decimal, Box<EvalAltResult>> {
+        parse_decimal(s)
     }
 
     pub fn sin(x: Decimal) -> Decimal {
@@ -480,39 +479,37 @@ mod decimal_functions {
     }
 
     #[cfg(not(feature = "no_float"))]
-    pub mod float {
-        use std::convert::TryFrom;
-
-        #[rhai_fn(name = "to_decimal", return_raw)]
-        pub fn f32_to_decimal(x: f32) -> Result<Decimal, Box<EvalAltResult>> {
-            Decimal::try_from(x).map_err(|_| {
-                EvalAltResult::ErrorArithmetic(
-                    format!("Cannot convert to Decimal: to_decimal({})", x),
-                    Position::NONE,
-                )
-                .into()
-            })
-        }
-        #[rhai_fn(name = "to_decimal", return_raw)]
-        pub fn f64_to_decimal(x: f64) -> Result<Decimal, Box<EvalAltResult>> {
-            Decimal::try_from(x).map_err(|_| {
-                EvalAltResult::ErrorArithmetic(
-                    format!("Cannot convert to Decimal: to_decimal({})", x),
-                    Position::NONE,
-                )
-                .into()
-            })
-        }
-        #[rhai_fn(return_raw)]
-        pub fn to_float(x: Decimal) -> Result<FLOAT, Box<EvalAltResult>> {
-            FLOAT::try_from(x).map_err(|_| {
-                EvalAltResult::ErrorArithmetic(
-                    format!("Cannot convert to floating-point: to_float({})", x),
-                    Position::NONE,
-                )
-                .into()
-            })
-        }
+    #[rhai_fn(name = "to_decimal", return_raw)]
+    pub fn f32_to_decimal(x: f32) -> Result<Decimal, Box<EvalAltResult>> {
+        Decimal::try_from(x).map_err(|_| {
+            EvalAltResult::ErrorArithmetic(
+                format!("Cannot convert to Decimal: to_decimal({})", x),
+                Position::NONE,
+            )
+            .into()
+        })
+    }
+    #[cfg(not(feature = "no_float"))]
+    #[rhai_fn(name = "to_decimal", return_raw)]
+    pub fn f64_to_decimal(x: f64) -> Result<Decimal, Box<EvalAltResult>> {
+        Decimal::try_from(x).map_err(|_| {
+            EvalAltResult::ErrorArithmetic(
+                format!("Cannot convert to Decimal: to_decimal({})", x),
+                Position::NONE,
+            )
+            .into()
+        })
+    }
+    #[cfg(not(feature = "no_float"))]
+    #[rhai_fn(return_raw)]
+    pub fn to_float(x: Decimal) -> Result<FLOAT, Box<EvalAltResult>> {
+        FLOAT::try_from(x).map_err(|_| {
+            EvalAltResult::ErrorArithmetic(
+                format!("Cannot convert to floating-point: to_float({})", x),
+                Position::NONE,
+            )
+            .into()
+        })
     }
 }
 
