@@ -4,6 +4,7 @@ use crate::{Module, Shared};
 
 pub(crate) mod arithmetic;
 mod array_basic;
+mod blob_basic;
 mod fn_basic;
 mod iter_basic;
 mod lang_core;
@@ -19,6 +20,8 @@ mod time_basic;
 pub use arithmetic::ArithmeticPackage;
 #[cfg(not(feature = "no_index"))]
 pub use array_basic::BasicArrayPackage;
+#[cfg(not(feature = "no_index"))]
+pub use blob_basic::BasicBlobPackage;
 pub use fn_basic::BasicFnPackage;
 pub use iter_basic::BasicIteratorPackage;
 pub use logic::LogicPackage;
@@ -34,10 +37,11 @@ pub use time_basic::BasicTimePackage;
 
 /// Trait that all packages must implement.
 pub trait Package {
-    /// Register all the functions in a package into a store.
-    fn init(lib: &mut Module);
+    /// Initialize the package.
+    /// Functions should be registered into `module` here.
+    fn init(module: &mut Module);
 
-    /// Retrieve the generic package library from this package.
+    /// Get a reference to a shared module from this package.
     #[must_use]
     fn as_shared_module(&self) -> Shared<Module>;
 }
@@ -57,10 +61,10 @@ pub trait Package {
 ///
 /// fn add(x: i64, y: i64) -> Result<i64, Box<EvalAltResult>> { Ok(x + y) }
 ///
-/// def_package!(rhai:MyPackage:"My super-duper package", lib,
+/// def_package!(rhai:MyPackage:"My super-duper package", module,
 /// {
 ///     // Load a binary function with all value parameters.
-///     lib.set_native_fn("my_add", add);
+///     module.set_native_fn("my_add", add);
 /// });
 /// ```
 #[macro_export]
@@ -73,7 +77,6 @@ macro_rules! def_package {
             fn as_shared_module(&self) -> $root::Shared<$root::Module> {
                 self.0.clone()
             }
-
             fn init($lib: &mut $root::Module) {
                 $block
             }
