@@ -9,6 +9,8 @@ use std::{any::TypeId, mem};
 use super::string_basic::{print_with_func, FUNC_TO_STRING};
 
 def_package!(crate:MoreStringPackage:"Additional string utilities, including string building.", lib, {
+    lib.standard = true;
+
     combine_with_exported_module!(lib, "string", string_functions);
 });
 
@@ -45,19 +47,28 @@ mod string_functions {
         s
     }
 
-    #[rhai_fn(name = "append")]
+    #[rhai_fn(name = "+", name = "append")]
     pub fn add_append_str(string1: ImmutableString, string2: ImmutableString) -> ImmutableString {
         string1 + string2
     }
-    #[rhai_fn(name = "append")]
+    #[rhai_fn(name = "+", name = "append")]
     pub fn add_append_char(string: ImmutableString, character: char) -> ImmutableString {
         string + character
     }
-    #[rhai_fn(name = "append")]
+    #[rhai_fn(name = "+")]
+    pub fn add_prepend_char(character: char, string: ImmutableString) -> ImmutableString {
+        format!("{}{}", character, string).into()
+    }
+
+    #[rhai_fn(name = "+", name = "append")]
     pub fn add_append_unit(string: ImmutableString, item: ()) -> ImmutableString {
         let _item = item;
         string
     }
+    #[rhai_fn(name = "+")]
+    pub fn add_prepend_unit(_item: (), string: ImmutableString) -> ImmutableString {
+        string
+    }
 
     #[rhai_fn(name = "len", get = "len")]
     pub fn len(string: &str) -> INT {
@@ -164,7 +175,7 @@ mod string_functions {
     #[rhai_fn(name = "to_upper")]
     pub fn to_upper_char(character: char) -> char {
         let mut stream = character.to_uppercase();
-        let ch = stream.next().expect("at least one character");
+        let ch = stream.next().expect("not empty");
         if stream.next().is_some() {
             character
         } else {
@@ -178,9 +189,7 @@ mod string_functions {
     #[rhai_fn(name = "to_lower")]
     pub fn to_lower_char(character: char) -> char {
         let mut stream = character.to_lowercase();
-        let ch = stream
-            .next()
-            .expect("there should be at least one character");
+        let ch = stream.next().expect("not empty");
         if stream.next().is_some() {
             character
         } else {
@@ -460,17 +469,16 @@ mod string_functions {
         if len <= 0 {
             return Ok(());
         }
-
         let _ctx = ctx;
 
         // Check if string will be over max size limit
         #[cfg(not(feature = "unchecked"))]
         if _ctx.engine().max_string_size() > 0 && len as usize > _ctx.engine().max_string_size() {
-            return crate::EvalAltResult::ErrorDataTooLarge(
+            return Err(crate::EvalAltResult::ErrorDataTooLarge(
                 "Length of string".to_string(),
                 crate::Position::NONE,
             )
-            .into();
+            .into());
         }
 
         let orig_len = string.chars().count();
@@ -485,11 +493,11 @@ mod string_functions {
             #[cfg(not(feature = "unchecked"))]
             if _ctx.engine().max_string_size() > 0 && string.len() > _ctx.engine().max_string_size()
             {
-                return crate::EvalAltResult::ErrorDataTooLarge(
+                return Err(crate::EvalAltResult::ErrorDataTooLarge(
                     "Length of string".to_string(),
                     crate::Position::NONE,
                 )
-                .into();
+                .into());
             }
         }
 
@@ -505,17 +513,16 @@ mod string_functions {
         if len <= 0 {
             return Ok(());
         }
-
         let _ctx = ctx;
 
         // Check if string will be over max size limit
         #[cfg(not(feature = "unchecked"))]
         if _ctx.engine().max_string_size() > 0 && len as usize > _ctx.engine().max_string_size() {
-            return crate::EvalAltResult::ErrorDataTooLarge(
+            return Err(crate::EvalAltResult::ErrorDataTooLarge(
                 "Length of string".to_string(),
                 crate::Position::NONE,
             )
-            .into();
+            .into());
         }
 
         let mut str_len = string.chars().count();
@@ -537,11 +544,11 @@ mod string_functions {
             #[cfg(not(feature = "unchecked"))]
             if _ctx.engine().max_string_size() > 0 && string.len() > _ctx.engine().max_string_size()
             {
-                return crate::EvalAltResult::ErrorDataTooLarge(
+                return Err(crate::EvalAltResult::ErrorDataTooLarge(
                     "Length of string".to_string(),
                     crate::Position::NONE,
                 )
-                .into();
+                .into());
             }
         }
 
