@@ -1,8 +1,11 @@
-use crate::{Engine, AST};
+#![cfg(feature = "metadata")]
+
+use crate::module::calc_native_fn_hash;
+use crate::{calc_fn_hash, Engine, AST};
 use serde::{Deserialize, Serialize};
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
-use std::{cmp::Ordering, collections::BTreeMap};
+use std::{cmp::Ordering, collections::BTreeMap, iter::empty};
 
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Hash, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
@@ -46,34 +49,28 @@ impl From<crate::FnAccess> for FnAccess {
 #[derive(Debug, Clone, Eq, PartialEq, Hash, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 struct FnParam {
-    pub name: String,
+    pub name: Box<str>,
     #[serde(rename = "type", skip_serializing_if = "Option::is_none")]
-    pub typ: Option<String>,
+    pub typ: Option<Box<str>>,
 }
 
 impl PartialOrd for FnParam {
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        Some(
-            match self
-                .name
-                .partial_cmp(&other.name)
-                .expect("String::partial_cmp should succeed")
-            {
-                Ordering::Less => Ordering::Less,
-                Ordering::Greater => Ordering::Greater,
-                Ordering::Equal => match (self.typ.is_none(), other.typ.is_none()) {
-                    (true, true) => Ordering::Equal,
-                    (true, false) => Ordering::Greater,
-                    (false, true) => Ordering::Less,
-                    (false, false) => self
-                        .typ
-                        .as_ref()
-                        .expect("`typ` is not `None`")
-                        .partial_cmp(other.typ.as_ref().expect("`typ` is not `None`"))
-                        .expect("String::partial_cmp should succeed"),
-                },
+        Some(match self.name.partial_cmp(&other.name).expect("succeed") {
+            Ordering::Less => Ordering::Less,
+            Ordering::Greater => Ordering::Greater,
+            Ordering::Equal => match (self.typ.is_none(), other.typ.is_none()) {
+                (true, true) => Ordering::Equal,
+                (true, false) => Ordering::Greater,
+                (false, true) => Ordering::Less,
+                (false, false) => self
+                    .typ
+                    .as_ref()
+                    .expect("`Some`")
+                    .partial_cmp(other.typ.as_ref().expect("`Some`"))
+                    .expect("succeed"),
             },
-        )
+        })
     }
 }
 
@@ -89,6 +86,8 @@ impl Ord for FnParam {
 #[derive(Debug, Clone, Eq, PartialEq, Hash, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 struct FnMetadata {
+    pub base_hash: u64,
+    pub full_hash: u64,
     pub namespace: FnNamespace,
     pub access: FnAccess,
     pub name: String,
@@ -98,10 +97,10 @@ struct FnMetadata {
     #[serde(default, skip_serializing_if = "Vec::is_empty")]
     pub params: Vec<FnParam>,
     #[serde(default, skip_serializing_if = "Option::is_none")]
-    pub return_type: Option<String>,
+    pub return_type: Option<Box<str>>,
     pub signature: String,
     #[serde(default, skip_serializing_if = "Vec::is_empty")]
-    pub doc_comments: Vec<String>,
+    pub doc_comments: Vec<Box<str>>,
 }
 
 impl PartialOrd for FnMetadata {
@@ -124,15 +123,23 @@ impl Ord for FnMetadata {
 
 impl From<&crate::module::FuncInfo> for FnMetadata {
     fn from(info: &crate::module::FuncInfo) -> Self {
+        let base_hash = calc_fn_hash(&info.name, info.params);
+        let (typ, full_hash) = if info.func.is_script() {
+            (FnType::Script, base_hash)
+        } else {
+            (
+                FnType::Native,
+                calc_native_fn_hash(empty::<&str>(), &info.name, &info.param_types),
+            )
+        };
+
         Self {
+            base_hash,
+            full_hash,
             namespace: info.namespace.into(),
             access: info.access.into(),
             name: info.name.to_string(),
-            typ: if info.func.is_script() {
-                FnType::Script
-            } else {
-                FnType::Native
-            },
+            typ,
             num_params: info.params,
             params: info
                 .param_names
@@ -142,17 +149,17 @@ impl From<&crate::module::FuncInfo> for FnMetadata {
                     let mut seg = s.splitn(2, ':');
                     let name = seg
                         .next()
-                        .map(|s| s.trim().to_string())
-                        .unwrap_or_else(|| "_".to_string());
-                    let typ = seg.next().map(|s| s.trim().to_string());
+                        .map(|s| s.trim().into())
+                        .unwrap_or_else(|| "_".into());
+                    let typ = seg.next().map(|s| s.trim().into());
                     FnParam { name, typ }
                 })
                 .collect(),
             return_type: info
                 .param_names
                 .last()
-                .map(|s| s.to_string())
-                .or_else(|| Some("()".to_string())),
+                .map(|s| s.as_str().into())
+                .or_else(|| Some("()".into())),
             signature: info.gen_signature(),
             doc_comments: if info.func.is_script() {
                 #[cfg(feature = "no_function")]
@@ -165,40 +172,17 @@ impl From<&crate::module::FuncInfo> for FnMetadata {
                         .get_script_fn_def()
                         .expect("scripted function")
                         .comments
-                        .to_vec()
+                        .as_ref()
+                        .map_or_else(|| Vec::new(), |v| v.to_vec())
                 }
             } else {
-                Default::default()
+                Vec::new()
             },
         }
     }
 }
 
-#[cfg(not(feature = "no_function"))]
-impl From<crate::ast::ScriptFnMetadata<'_>> for FnMetadata {
-    fn from(info: crate::ast::ScriptFnMetadata) -> Self {
-        Self {
-            namespace: FnNamespace::Global,
-            access: info.access.into(),
-            name: info.name.to_string(),
-            typ: FnType::Script,
-            num_params: info.params.len(),
-            params: info
-                .params
-                .iter()
-                .map(|s| FnParam {
-                    name: s.to_string(),
-                    typ: Some("Dynamic".to_string()),
-                })
-                .collect(),
-            return_type: Some("Dynamic".to_string()),
-            signature: info.to_string(),
-            doc_comments: info.comments.iter().map(|s| s.to_string()).collect(),
-        }
-    }
-}
-
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Debug, Clone, Serialize)]
 #[serde(rename_all = "camelCase")]
 struct ModuleMetadata {
     #[serde(skip_serializing_if = "BTreeMap::is_empty")]
@@ -210,7 +194,10 @@ struct ModuleMetadata {
 impl ModuleMetadata {
     #[inline(always)]
     pub fn new() -> Self {
-        Default::default()
+        Self {
+            modules: BTreeMap::new(),
+            functions: Vec::new(),
+        }
     }
 }
 
@@ -265,7 +252,8 @@ impl Engine {
             .for_each(|f| global.functions.push(f.into()));
 
         #[cfg(not(feature = "no_function"))]
-        _ast.iter_functions()
+        _ast.shared_lib()
+            .iter_fn()
             .for_each(|f| global.functions.push(f.into()));
 
         global.functions.sort();
@@ -274,13 +262,14 @@ impl Engine {
     }
 
     /// Generate a list of all functions in JSON format.
-    /// Available only under the `metadata` feature.
+    /// Exported under the `metadata` feature only.
     ///
     /// Functions from the following sources are included:
     /// 1) Functions registered into the global namespace
     /// 2) Functions in static modules
     /// 3) Functions in global modules (optional)
+    #[inline(always)]
     pub fn gen_fn_metadata_to_json(&self, include_global: bool) -> serde_json::Result<String> {
-        self.gen_fn_metadata_with_ast_to_json(&Default::default(), include_global)
+        self.gen_fn_metadata_with_ast_to_json(&AST::empty(), include_global)
     }
 }
