@@ -9,12 +9,6 @@ use std::fmt;
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
 
-#[cfg(not(feature = "no_index"))]
-use crate::Array;
-
-#[cfg(not(feature = "no_object"))]
-use crate::Map;
-
 /// Serializer for [`Dynamic`][crate::Dynamic] which is kept as a reference.
 struct DynamicSerializer {
     /// Buffer to hold a temporary key.
@@ -28,7 +22,7 @@ impl DynamicSerializer {
     #[must_use]
     pub fn new(_value: Dynamic) -> Self {
         Self {
-            _key: Default::default(),
+            _key: Dynamic::UNIT,
             _value,
         }
     }
@@ -83,7 +77,7 @@ impl DynamicSerializer {
 /// # }
 /// ```
 pub fn to_dynamic<T: Serialize>(value: T) -> RhaiResult {
-    let mut s = DynamicSerializer::new(Default::default());
+    let mut s = DynamicSerializer::new(Dynamic::UNIT);
     value.serialize(&mut s)
 }
 
@@ -257,11 +251,20 @@ impl Serializer for &mut DynamicSerializer {
     }
 
     fn serialize_str(self, v: &str) -> Result<Self::Ok, Box<EvalAltResult>> {
-        Ok(v.to_string().into())
+        Ok(v.into())
     }
 
-    fn serialize_bytes(self, v: &[u8]) -> Result<Self::Ok, Box<EvalAltResult>> {
-        Ok(Dynamic::from(v.to_vec()))
+    fn serialize_bytes(self, _v: &[u8]) -> Result<Self::Ok, Box<EvalAltResult>> {
+        #[cfg(not(feature = "no_index"))]
+        return Ok(Dynamic::from_blob(_v.to_vec()));
+
+        #[cfg(feature = "no_index")]
+        return Err(EvalAltResult::ErrorMismatchDataType(
+            "".into(),
+            "BLOB's are not supported with 'no_index'".into(),
+            Position::NONE,
+        )
+        .into());
     }
 
     fn serialize_none(self) -> Result<Self::Ok, Box<EvalAltResult>> {
@@ -313,24 +316,24 @@ impl Serializer for &mut DynamicSerializer {
             make_variant(_variant, content)
         }
         #[cfg(feature = "no_object")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "object maps are not supported with 'no_object'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 
     fn serialize_seq(self, _len: Option<usize>) -> Result<Self::SerializeSeq, Box<EvalAltResult>> {
         #[cfg(not(feature = "no_index"))]
-        return Ok(DynamicSerializer::new(Array::new().into()));
+        return Ok(DynamicSerializer::new(crate::Array::new().into()));
         #[cfg(feature = "no_index")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "arrays are not supported with 'no_index'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 
     fn serialize_tuple(self, len: usize) -> Result<Self::SerializeTuple, Box<EvalAltResult>> {
@@ -356,27 +359,27 @@ impl Serializer for &mut DynamicSerializer {
         #[cfg(not(feature = "no_index"))]
         return Ok(TupleVariantSerializer {
             variant: _variant,
-            array: Array::with_capacity(_len),
+            array: crate::Array::with_capacity(_len),
         });
         #[cfg(any(feature = "no_object", feature = "no_index"))]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "tuples are not supported with 'no_index' or 'no_object'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 
     fn serialize_map(self, _len: Option<usize>) -> Result<Self::SerializeMap, Box<EvalAltResult>> {
         #[cfg(not(feature = "no_object"))]
-        return Ok(DynamicSerializer::new(Map::new().into()));
+        return Ok(DynamicSerializer::new(crate::Map::new().into()));
         #[cfg(feature = "no_object")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "object maps are not supported with 'no_object'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 
     fn serialize_struct(
@@ -397,15 +400,15 @@ impl Serializer for &mut DynamicSerializer {
         #[cfg(not(feature = "no_object"))]
         return Ok(StructVariantSerializer {
             variant: _variant,
-            map: Default::default(),
+            map: crate::Map::new(),
         });
         #[cfg(feature = "no_object")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "object maps are not supported with 'no_object'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 }
 
@@ -420,17 +423,17 @@ impl SerializeSeq for DynamicSerializer {
         #[cfg(not(feature = "no_index"))]
         {
             let _value = _value.serialize(&mut *self)?;
-            let arr = self._value.downcast_mut::<Array>().unwrap();
+            let arr = self._value.downcast_mut::<crate::Array>().unwrap();
             arr.push(_value);
             Ok(())
         }
         #[cfg(feature = "no_index")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "arrays are not supported with 'no_index'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 
     // Close the sequence.
@@ -438,12 +441,12 @@ impl SerializeSeq for DynamicSerializer {
         #[cfg(not(feature = "no_index"))]
         return Ok(self._value);
         #[cfg(feature = "no_index")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "arrays are not supported with 'no_index'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 }
 
@@ -458,29 +461,29 @@ impl SerializeTuple for DynamicSerializer {
         #[cfg(not(feature = "no_index"))]
         {
             let _value = _value.serialize(&mut *self)?;
-            let arr = self._value.downcast_mut::<Array>().unwrap();
+            let arr = self._value.downcast_mut::<crate::Array>().unwrap();
             arr.push(_value);
             Ok(())
         }
         #[cfg(feature = "no_index")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "tuples are not supported with 'no_index'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 
     fn end(self) -> Result<Self::Ok, Box<EvalAltResult>> {
         #[cfg(not(feature = "no_index"))]
         return Ok(self._value);
         #[cfg(feature = "no_index")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "tuples are not supported with 'no_index'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 }
 
@@ -495,29 +498,29 @@ impl SerializeTupleStruct for DynamicSerializer {
         #[cfg(not(feature = "no_index"))]
         {
             let _value = _value.serialize(&mut *self)?;
-            let arr = self._value.downcast_mut::<Array>().unwrap();
+            let arr = self._value.downcast_mut::<crate::Array>().unwrap();
             arr.push(_value);
             Ok(())
         }
         #[cfg(feature = "no_index")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "tuples are not supported with 'no_index'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 
     fn end(self) -> Result<Self::Ok, Box<EvalAltResult>> {
         #[cfg(not(feature = "no_index"))]
         return Ok(self._value);
         #[cfg(feature = "no_index")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "tuples are not supported with 'no_index'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 }
 
@@ -532,12 +535,12 @@ impl SerializeMap for DynamicSerializer {
             Ok(())
         }
         #[cfg(feature = "no_object")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "object maps are not supported with 'no_object'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 
     fn serialize_value<T: ?Sized + Serialize>(
@@ -556,17 +559,17 @@ impl SerializeMap for DynamicSerializer {
                     )
                 })?;
             let _value = _value.serialize(&mut *self)?;
-            let map = self._value.downcast_mut::<Map>().unwrap();
+            let map = self._value.downcast_mut::<crate::Map>().unwrap();
             map.insert(key.into(), _value);
             Ok(())
         }
         #[cfg(feature = "no_object")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "object maps are not supported with 'no_object'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 
     fn serialize_entry<K: ?Sized + Serialize, T: ?Sized + Serialize>(
@@ -581,29 +584,29 @@ impl SerializeMap for DynamicSerializer {
                 EvalAltResult::ErrorMismatchDataType("string".into(), typ.into(), Position::NONE)
             })?;
             let _value = _value.serialize(&mut *self)?;
-            let map = self._value.downcast_mut::<Map>().unwrap();
+            let map = self._value.downcast_mut::<crate::Map>().unwrap();
             map.insert(_key.into(), _value);
             Ok(())
         }
         #[cfg(feature = "no_object")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "object maps are not supported with 'no_object'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 
     fn end(self) -> Result<Self::Ok, Box<EvalAltResult>> {
         #[cfg(not(feature = "no_object"))]
         return Ok(self._value);
         #[cfg(feature = "no_object")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "object maps are not supported with 'no_object'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 }
 
@@ -619,36 +622,36 @@ impl SerializeStruct for DynamicSerializer {
         #[cfg(not(feature = "no_object"))]
         {
             let _value = _value.serialize(&mut *self)?;
-            let map = self._value.downcast_mut::<Map>().unwrap();
+            let map = self._value.downcast_mut::<crate::Map>().unwrap();
             map.insert(_key.into(), _value);
             Ok(())
         }
         #[cfg(feature = "no_object")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "object maps are not supported with 'no_object'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 
     fn end(self) -> Result<Self::Ok, Box<EvalAltResult>> {
         #[cfg(not(feature = "no_object"))]
         return Ok(self._value);
         #[cfg(feature = "no_object")]
-        return EvalAltResult::ErrorMismatchDataType(
+        return Err(EvalAltResult::ErrorMismatchDataType(
             "".into(),
             "object maps are not supported with 'no_object'".into(),
             Position::NONE,
         )
-        .into();
+        .into());
     }
 }
 
 #[cfg(not(any(feature = "no_object", feature = "no_index")))]
 struct TupleVariantSerializer {
     variant: &'static str,
-    array: Array,
+    array: crate::Array,
 }
 
 #[cfg(not(any(feature = "no_object", feature = "no_index")))]
@@ -673,7 +676,7 @@ impl serde::ser::SerializeTupleVariant for TupleVariantSerializer {
 #[cfg(not(feature = "no_object"))]
 struct StructVariantSerializer {
     variant: &'static str,
-    map: Map,
+    map: crate::Map,
 }
 
 #[cfg(not(feature = "no_object"))]
@@ -698,7 +701,7 @@ impl serde::ser::SerializeStructVariant for StructVariantSerializer {
 
 #[cfg(not(feature = "no_object"))]
 fn make_variant(variant: &'static str, value: Dynamic) -> RhaiResult {
-    let mut map = Map::new();
+    let mut map = crate::Map::new();
     map.insert(variant.into(), value);
     Ok(map.into())
 }
